#!/usr/bin/python3

import io
import os
import tempfile
import unittest

import apt_pkg
import testcommon

import aptsources.distro
import aptsources.sourceslist


class TestAptSources(testcommon.TestCase):
    def setUp(self):
        testcommon.TestCase.setUp(self)
        self.tempfile = tempfile.NamedTemporaryFile(suffix=".sources")
        apt_pkg.config.set("Dir::Etc::sourcelist", self.tempfile.name)
        apt_pkg.config.set("Dir::Etc::sourceparts", "/dev/null")

    def tearDown(self):
        self.tempfile.close()

    def testEmptyDeb822(self):
        """aptsources: Test sources.list parsing."""
        sources = aptsources.sourceslist.SourcesList(True)
        self.assertListEqual(sources.list, [])

    def testDeb822SectionRecognizedWithoutEndLine(self):
        """aptsources: Test sources.list parsing."""
        section = aptsources._deb822.Section("key: value\notherkey: othervalue")

        # Writing it back out gives us an extra newline at the end
        self.assertEqual(section["key"], "value")
        self.assertEqual(section["otherkey"], "othervalue")
        self.assertEqual(str(section), "key: value\notherkey: othervalue\n")

        file = aptsources._deb822.File(io.StringIO("key: value\notherkey: othervalue"))
        self.assertEqual(len(file.sections), 1)

        section = next(iter(file))
        self.assertEqual(section["key"], "value")
        self.assertEqual(section["otherkey"], "othervalue")
        self.assertEqual(str(section), "key: value\notherkey: othervalue\n")

    def testDeb822CommentBlockWithoutEndLine(self):
        """aptsources: Test sources.list parsing."""
        section = aptsources._deb822.Section("# a comment\n# another comment")

        self.assertEqual(section.header, " a comment\n another comment\n")
        # Writing it back out gives us an extra newline at the end
        self.assertEqual(str(section), "# a comment\n# another comment\n")

    def testDeb822MultipleLinesSeparator(self):
        """aptsources: Test sources.list parsing."""
        for separator in "\n\n\n\n", "\n\n\n", "\n\n":
            with self.subTest(f"{len(separator)} separators"):
                file = aptsources._deb822.File(
                    io.StringIO("key: value" + separator + "otherkey: othervalue\n")
                )
                self.assertEqual(len(file.sections), 2)

                self.assertEqual(file.sections[0]["key"], "value")
                self.assertEqual(file.sections[1]["otherkey"], "othervalue")
                self.assertEqual(str(file), "key: value\n\notherkey: othervalue\n")


if __name__ == "__main__":
    os.chdir(os.path.dirname(__file__))
    unittest.main()



#!/usr/bin/python3

import copy
import os
import tempfile
import unittest

import apt_pkg
import testcommon

import aptsources.distro
import aptsources.sourceslist


class TestAptSources(testcommon.TestCase):
    def setUp(self):
        testcommon.TestCase.setUp(self)
        if apt_pkg.config["APT::Architecture"] not in ("i386", "amd64"):
            apt_pkg.config.set("APT::Architecture", "i386")
        apt_pkg.config.set("Dir::Etc", os.getcwd())
        apt_pkg.config.set("Dir::Etc::sourceparts", tempfile.mkdtemp())
        if os.path.exists("./build/data/templates"):
            self.templates = os.path.abspath("./build/data/templates")
        elif os.path.exists("../build/data/templates"):
            self.templates = os.path.abspath("../build/data/templates")
        else:
            self.templates = "/usr/share/python-apt/templates/"

    def testIsMirror(self):
        """aptsources: Test mirror detection."""
        yes = aptsources.sourceslist.is_mirror(
            "http://archive.ubuntu.com", "http://de.archive.ubuntu.com"
        )
        no = aptsources.sourceslist.is_mirror(
            "http://archive.ubuntu.com", "http://ftp.debian.org"
        )
        self.assertTrue(yes)
        self.assertFalse(no)

    def testSourcesListReading(self):
        """aptsources: Test sources.list parsing."""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        self.assertEqual(len(sources.list), 10)
        # test load
        sources.list = []
        sources.load("data/aptsources/sources.list")
        self.assertEqual(len(sources.list), 10)

    def testSourcesListReading_deb822(self):
        """aptsources: Test sources.list parsing."""
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        self.assertEqual(len(sources.list), 5)
        # test load
        sources.list = []
        sources.load("data/aptsources/sources.list.d/main.sources")
        self.assertEqual(len(sources.list), 5)

        for entry in sources.list:
            self.assertFalse(entry.invalid)
            self.assertFalse(entry.disabled)
            self.assertEqual(entry.types, ["deb"])
            self.assertEqual(entry.type, "deb")
            self.assertEqual(entry.uris, ["http://de.archive.ubuntu.com/ubuntu/"])
            self.assertEqual(entry.uri, "http://de.archive.ubuntu.com/ubuntu/")

        self.assertEqual(sources.list[0].comps, ["main"])
        self.assertEqual(sources.list[1].comps, ["restricted"])
        self.assertEqual(sources.list[2].comps, ["universe"])
        self.assertEqual(sources.list[3].comps, ["main"])
        self.assertEqual(sources.list[4].comps, ["main"])

        for entry in sources.list[:-1]:
            self.assertIsNone(entry.trusted)

        self.assertTrue(sources.list[-1].trusted)

        for entry in sources.list[:-2]:
            self.assertEqual(entry.architectures, [])
            self.assertEqual(entry.suites, ["edgy"])
            self.assertEqual(entry.dist, "edgy")

        for entry in sources.list[-2:]:
            self.assertEqual(entry.suites, ["natty"])
            self.assertEqual(entry.dist, "natty")
            self.assertEqual(entry.architectures, ["amd64", "i386"])

    def testSourcesListWriting_deb822(self):
        """aptsources: Test sources.list parsing."""
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        with tempfile.TemporaryDirectory() as tmpdir:
            for entry in sources.list:
                entry.file = os.path.join(tmpdir, os.path.basename(entry.file))

            sources.save()

            maxDiff = self.maxDiff
            self.maxDiff = None
            for file in os.listdir("data/aptsources/sources.list.d"):
                with open(os.path.join("data/aptsources/sources.list.d", file)) as a:
                    with open(os.path.join(tmpdir, file)) as b:
                        self.assertEqual(a.read(), b.read(), f"file {file}")
            self.maxDiff = maxDiff

    def testSourcesListAdding(self):
        """aptsources: Test additions to sources.list"""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        # test to add something that is already there (main)
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["main"])
        self.assertTrue(sources.list == before.list)
        # test to add something that is already there (restricted)
        before = copy.deepcopy(sources)
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["restricted"]
        )
        self.assertTrue(sources.list == before.list)

        before = copy.deepcopy(sources)
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "natty",
            ["main"],
            architectures=["amd64", "i386"],
        )
        self.assertTrue(sources.list == before.list)

        # test to add something new: multiverse
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["multiverse"]
        )
        found = False
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "edgy"
                and "multiverse" in entry.comps
            ):
                found = True
                break
        self.assertTrue(found)

        # add a new natty entry without architecture specification
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "natty", ["multiverse"]
        )
        found = False
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "natty"
                and entry.architectures == []
                and "multiverse" in entry.comps
            ):
                found = True
                break
        self.assertTrue(found)

        # Add universe to existing multi-arch line
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "natty",
            ["universe"],
            architectures=["i386", "amd64"],
        )
        found = False
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "natty"
                and set(entry.architectures) == {"amd64", "i386"}
                and set(entry.comps) == {"main", "universe"}
            ):
                found = True
                break
        self.assertTrue(found)
        # test to add something new: multiverse *and*
        # something that is already there
        before = copy.deepcopy(sources)
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "edgy",
            ["universe", "something"],
        )
        found_universe = 0
        found_something = 0
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "edgy"
            ):
                for c in entry.comps:
                    if c == "universe":
                        found_universe += 1
                    if c == "something":
                        found_something += 1
        # print "\n".join([s.str() for s in sources])
        self.assertEqual(found_something, 1)
        self.assertEqual(found_universe, 1)

    def testSourcesListAdding_deb822(self):
        """aptsources: Test additions to sources.list"""
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")

        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        # test to add something that is already there (main)
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["main"])
        self.assertEqual(sources.list, before.list)
        # test to add something that is already there (restricted)
        before = copy.deepcopy(sources)
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["restricted"]
        )
        self.assertTrue(sources.list == before.list)

        before = copy.deepcopy(sources)
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "natty",
            ["main"],
            architectures=["amd64", "i386"],
        )
        self.assertEqual(
            [e.str() for e in sources.list], [e.str() for e in before.list]
        )

        # test to add something new: multiverse
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["multiverse"]
        )
        found = False
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "edgy"
                and "multiverse" in entry.comps
            ):
                found = True
                break
        self.assertTrue(found)

        # add a new natty entry without architecture specification
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "natty", ["multiverse"]
        )
        found = False
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "natty"
                and entry.architectures == []
                and "multiverse" in entry.comps
            ):
                found = True
                break
        self.assertTrue(found)

        # Add universe to existing multi-arch line
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "natty",
            ["universe"],
            architectures=["i386", "amd64"],
        )
        found = False
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "natty"
                and set(entry.architectures) == {"amd64", "i386"}
                and set(entry.comps) == {"main", "universe"}
            ):
                found = True
                break
        self.assertTrue(found)
        # test to add something new: multiverse *and*
        # something that is already there
        before = copy.deepcopy(sources)
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "edgy",
            ["universe", "something"],
        )
        found_universe = 0
        found_something = 0
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "edgy"
            ):
                for c in entry.comps:
                    if c == "universe":
                        found_universe += 1
                    if c == "something":
                        found_something += 1
        # print "\n".join([s.str() for s in sources])
        self.assertEqual(found_something, 1)
        self.assertEqual(found_universe, 1)

    def testAddingWithComment_deb822(self):
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        self._commonTestAddingWithComment()

    def testAddingWithComment_short(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        self._commonTestAddingWithComment()

    def _commonTestAddingWithComment(self):
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)

        # test to add something that is already there (main); loses comment
        before = copy.deepcopy(sources)
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "edgy",
            ["main"],
            comment="this will be lost",
        )
        self.assertTrue(sources.list == before.list)
        for entry in sources:
            self.assertNotEqual(entry.comment, "this will be lost")

        # test to add component to existing entry: multiverse; loses comment
        sources.add(
            "deb",
            "http://de.archive.ubuntu.com/ubuntu/",
            "edgy",
            ["multiverse"],
            comment="this will be lost",
        )
        for entry in sources:
            self.assertNotEqual(entry.comment, "this will be lost")

        before = copy.deepcopy(sources)
        # test to add entirely new entry; retains comment
        sources.add(
            "deb-src",
            "http://de.archive.ubuntu.com/ubuntu/",
            "edgy",
            ["main"],
            comment="this will appear",
            file=sources.list[0].file,  # make sure we test the deb822 code path
        )
        self.assertNotEqual(sources.list, before.list)
        self.assertEqual(len(sources.list), len(before.list) + 1)
        found = False
        for entry in sources:
            if (
                entry.type == "deb-src"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and entry.dist == "edgy"
                and entry.comment == "this will appear"
                and "main" in entry.comps
            ):
                found = True
                break
        self.assertTrue(found)

    def testInsertion_deb822(self):
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)

        # test to insert something that is already there (universe); does not
        # move existing entry (remains at index 2)
        before = copy.deepcopy(sources)
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["universe"], pos=0
        )
        self.assertTrue(sources.list == before.list)
        entry = list(sources)[2]
        self.assertEqual(entry.type, "deb")
        self.assertEqual(entry.uri, "http://de.archive.ubuntu.com/ubuntu/")
        self.assertEqual(entry.dist, "edgy")
        self.assertIn("universe", entry.comps)

        # test add component to existing entry: multiverse; does not move
        # entry to which it is appended (remains at index 0)
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["multiverse"], pos=2
        )
        entry = list(sources)[0]
        self.assertTrue(
            entry.type == "deb"
            and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
            and entry.dist == "edgy"
            and {"main", "multiverse"} <= set(entry.comps)
        )

        # test to add entirely new entry; inserted at 0
        sources.add(
            "deb-src", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["main"], pos=0
        )
        entry = list(sources)[0]
        self.assertTrue(
            entry.type == "deb-src"
            and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
            and entry.dist == "edgy"
            and "main" in entry.comps
        )

    def testInsertion(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)

        # test to insert something that is already there (universe); does not
        # move existing entry (remains at index 2)
        before = copy.deepcopy(sources)
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["universe"], pos=0
        )
        self.assertTrue(sources.list == before.list)
        entry = list(sources)[5]
        self.assertTrue(
            entry.type == "deb"
            and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
            and entry.dist == "edgy"
            and "universe" in entry.comps
        )

        # test add component to existing entry: multiverse; does not move
        # entry to which it is appended (remains at index 0)
        sources.add(
            "deb", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["multiverse"], pos=2
        )
        entry = list(sources)[1]
        self.assertTrue(
            entry.type == "deb"
            and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
            and entry.dist == "edgy"
            and {"main", "multiverse"} <= set(entry.comps)
        )

        # test to add entirely new entry; inserted at 0
        sources.add(
            "deb-src", "http://de.archive.ubuntu.com/ubuntu/", "edgy", ["main"], pos=0
        )
        entry = list(sources)[0]
        self.assertTrue(
            entry.type == "deb-src"
            and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
            and entry.dist == "edgy"
            and "main" in entry.comps
        )

    def testDuplication_short(self):
        apt_pkg.config.set(
            "Dir::Etc::sourcelist", "data/aptsources/sources.list.testDuplication"
        )
        return self.commonTestDuplication()

    def testDuplication_deb822(self):
        apt_pkg.config.set(
            "Dir::Etc::sourceparts", "data/aptsources/sources.list.d.testDuplication"
        )
        return self.commonTestDuplication()

    def commonTestDuplication(self):
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        test_url = "http://ppa.launchpad.net/me/myproject/ubuntu"
        # test to add something that is already there (enabled)
        before = copy.deepcopy(sources)
        sources.add("deb", test_url, "xenial", ["main"])
        self.assertEqual(sources.list, before.list)
        # test to add something that is already there (disabled)
        sources.add("# deb-src", test_url, "xenial", ["main"])
        self.assertEqual(sources.list, before.list)
        # test to enable something that is already there
        sources.add("deb-src", test_url, "xenial", ["main"])
        found = False
        self.assertEqual(len(sources.list), 2)
        for entry in sources:
            if (
                entry.type == "deb-src"
                and not entry.disabled
                and entry.uri == test_url
                and entry.dist == "xenial"
                and entry.architectures == []
                and entry.comps == ["main"]
            ):
                found = True
                break
        self.assertTrue(found)

    def testMatcher_short(self):
        """aptsources: Test matcher"""
        apt_pkg.config.set(
            "Dir::Etc::sourcelist", "data/aptsources/" "sources.list.testDistribution"
        )
        return self.commonTestMatcher()

    def testMatcher_deb822(self):
        """aptsources: Test matcher"""
        apt_pkg.config.set(
            "Dir::Etc::sourceparts",
            "data/aptsources/" "sources.list.d.testDistribution",
        )
        return self.commonTestMatcher()

    def commonTestMatcher(self):
        """aptsources: Test matcher"""
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        distro = aptsources.distro.get_distro(
            id="Ubuntu",
            codename="bionic",
            description="Ubuntu 18.04 LTS",
            release="18.04",
        )
        distro.get_sources(sources)
        # test if all suits of the current distro were detected correctly
        self.assertNotEqual(sources.list, [])
        for s in sources:
            if not s.template:
                self.fail("source entry '%s' has no matcher" % s)

        # Hack in a check for splitting of fields here.
        if sources.list[-1].file.endswith(".sources"):
            self.assertEqual(
                sources.list[-1].uris, ["cdrom:[Ubuntu 8.04 _Hardy Heron_]"]
            )
        self.assertEqual(sources.list[-1].uri, "cdrom:[Ubuntu 8.04 _Hardy Heron_]")

    def testMultiArch(self):
        """aptsources: Test multi-arch parsing"""

        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        assert not sources.list[8].invalid
        assert sources.list[8].type == "deb"
        assert sources.list[8].architectures == ["amd64", "i386"]
        assert sources.list[8].uri == "http://de.archive.ubuntu.com/ubuntu/"
        assert sources.list[8].dist == "natty"
        assert sources.list[8].comps == ["main"]
        assert sources.list[8].line.strip() == str(sources.list[8])
        assert sources.list[8].trusted is None

    def testMultiArch_deb822(self):
        """aptsources: Test multi-arch parsing"""

        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        assert not sources.list[3].invalid
        assert sources.list[3].type == "deb"
        assert sources.list[3].architectures == ["amd64", "i386"]
        assert sources.list[3].uri == "http://de.archive.ubuntu.com/ubuntu/"
        assert sources.list[3].dist == "natty"
        assert sources.list[3].comps == ["main"]
        self.assertEqual(sources.list[3].line.strip(), str(sources.list[3]))
        self.assertIsNone(sources.list[3].trusted)

    def testMultipleOptions(self):
        """aptsources: Test multi-arch parsing"""

        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        assert sources.list[9].invalid is False
        assert sources.list[9].type == "deb"
        assert sources.list[9].architectures == ["amd64", "i386"]
        self.assertEqual(sources.list[9].uri, "http://de.archive.ubuntu.com/ubuntu/")
        assert sources.list[9].dist == "natty"
        assert sources.list[9].comps == ["main"]
        assert sources.list[9].trusted
        assert sources.list[9].line.strip() == str(sources.list[9])

    def testMultipleOptions_deb822(self):
        """aptsources: Test multi-arch parsing"""

        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        assert sources.list[4].invalid is False
        assert sources.list[4].type == "deb"
        assert sources.list[4].architectures == ["amd64", "i386"]
        self.assertEqual(sources.list[4].uri, "http://de.archive.ubuntu.com/ubuntu/")
        assert sources.list[4].dist == "natty"
        assert sources.list[4].comps == ["main"]
        assert sources.list[4].trusted
        assert sources.list[4].line.strip() == str(sources.list[4])

    def test_enable_component(self):
        target = "./data/aptsources/sources.list.enable_comps"
        line = "deb http://archive.ubuntu.com/ubuntu lucid main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        apt_pkg.config.set("Dir::Etc::sourcelist", target)
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        distro = aptsources.distro.get_distro(
            id="Ubuntu", codename="lucid", release="10.04", description="Ubuntu 10.04"
        )
        # and get the sources
        distro.get_sources(sources)
        # test enable_component
        comp = "multiverse"
        distro.enable_component(comp)
        comps = set()
        for entry in sources:
            comps = comps.union(set(entry.comps))
        self.assertTrue("multiverse" in comps)
        self.assertTrue("universe" in comps)

    def test_enable_component_deb822(self):
        target = (
            apt_pkg.config.find_dir("dir::etc::sourceparts") + "enable_comps.sources"
        )
        line = "Types: deb\nURIs: http://archive.ubuntu.com/ubuntu\nSuites: lucid\nComponents: main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        distro = aptsources.distro.get_distro(
            id="Ubuntu", codename="lucid", release="10.04", description="Ubuntu 10.04"
        )
        # and get the sources
        distro.get_sources(sources)
        # test enable_component
        comp = "multiverse"
        distro.enable_component(comp)
        comps = set()
        for entry in sources:
            comps = comps.union(set(entry.comps))
        self.assertTrue("multiverse" in comps)
        self.assertTrue("universe" in comps)

    def test_enable_component_deb822_multi(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "/dev/null")

        target = (
            apt_pkg.config.find_dir("dir::etc::sourceparts") + "enable_comps.sources"
        )
        line = "Types: deb\nURIs: http://archive.ubuntu.com/ubuntu\nSuites: lucid lucid-updates\nComponents: main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        distro = aptsources.distro.get_distro(
            id="Ubuntu", codename="lucid", release="10.04", description="Ubuntu 10.04"
        )
        # and get the sources
        distro.get_sources(sources)
        self.assertEqual(len(distro.main_sources), 1)
        self.assertEqual(len(sources.list), 1)
        # test enable_component
        comp = "multiverse"
        distro.enable_component(comp)
        self.assertEqual(len(sources.list), 2)  # split into two

        self.assertEqual(
            "Types: deb\n"
            "URIs: http://archive.ubuntu.com/ubuntu\n"
            "Suites: lucid\n"
            "Components: main multiverse universe",
            str(sources.list[0]),
        )
        self.assertEqual(
            "Types: deb\n"
            "URIs: http://archive.ubuntu.com/ubuntu\n"
            "Suites: lucid-updates\n"
            "Components: main multiverse universe",
            str(sources.list[1]),
        )

        comps = set()
        for entry in sources:
            comps = comps.union(set(entry.comps))
        self.assertTrue("multiverse" in comps)
        self.assertTrue("universe" in comps)

        sources.save()
        self.assertEqual(
            "Types: deb\n"
            "URIs: http://archive.ubuntu.com/ubuntu\n"
            "Suites: lucid lucid-updates\n"
            "Components: main multiverse universe",
            str(sources.list[0]),
        )

    def test_enable_component_deb822_multi_mixed_origin(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "/dev/null")

        target = (
            apt_pkg.config.find_dir("dir::etc::sourceparts") + "enable_comps.sources"
        )
        line = "Types: deb\nURIs: http://archive.ubuntu.com/ubuntu http://example.com/\nSuites: lucid\nComponents: main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        distro = aptsources.distro.get_distro(
            id="Ubuntu", codename="lucid", release="10.04", description="Ubuntu 10.04"
        )
        # and get the sources
        distro.get_sources(sources)
        self.assertEqual(len(distro.main_sources), 2)
        self.assertEqual(len(sources.list), 1)
        # test enable_component
        comp = "multiverse"
        distro.enable_component(comp)
        self.assertEqual(len(sources.list), 2)  # split into two

        self.assertEqual(
            "Types: deb\n"
            "URIs: http://archive.ubuntu.com/ubuntu\n"
            "Suites: lucid\n"
            "Components: main multiverse universe",
            str(sources.list[0]),
        )
        self.assertEqual(
            "Types: deb\n"
            "URIs: http://example.com/\n"
            "Suites: lucid\n"
            "Components: main",
            str(sources.list[1]),
        )

        sources.save()
        self.assertEqual(
            "Types: deb\n"
            "URIs: http://archive.ubuntu.com/ubuntu\n"
            "Suites: lucid\n"
            "Components: main multiverse universe",
            str(sources.list[0]),
        )
        self.assertEqual(
            "Types: deb\n"
            "URIs: http://example.com/\n"
            "Suites: lucid\n"
            "Components: main",
            str(sources.list[1]),
        )

    def test_enable_component_deb822_multi_mixed_ultimate(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "/dev/null")

        target = (
            apt_pkg.config.find_dir("dir::etc::sourceparts") + "enable_comps.sources"
        )
        line = "Types: deb deb-src\nURIs: http://archive.ubuntu.com/ubuntu http://example.com/\nSuites: lucid lucid-updates notalucid\nComponents: main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        distro = aptsources.distro.get_distro(
            id="Ubuntu", codename="lucid", release="10.04", description="Ubuntu 10.04"
        )
        # and get the sources
        distro.get_sources(sources)
        self.assertEqual(len(distro.main_sources), 1)
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 12)
        # test enable_component
        comp = "multiverse"
        distro.enable_component(comp)
        self.assertEqual(len(sources.list), 12)  # split into two

        expected = []
        for typ in "deb", "deb-src":
            for uri in "http://archive.ubuntu.com/ubuntu", "http://example.com/":
                for suite in "lucid", "lucid-updates", "notalucid":
                    comps = "main multiverse universe"
                    # unofficial source ends up without enablement
                    if uri == "http://example.com/" or suite == "notalucid":
                        comps = "main"
                    expected.append(
                        f"Types: {typ}\n"
                        f"URIs: {uri}\n"
                        f"Suites: {suite}\n"
                        f"Components: {comps}"
                    )

        self.maxDiff = None
        self.assertEqual(expected, list(map(str, sources.list)))
        sources.save()

        expected = [
            "Types: deb deb-src\n"
            "URIs: http://archive.ubuntu.com/ubuntu\n"
            "Suites: lucid lucid-updates\n"
            "Components: main multiverse universe",
            # unofficial suite
            "Types: deb deb-src\n"
            "URIs: http://archive.ubuntu.com/ubuntu http://example.com/\n"
            "Suites: notalucid\n"
            "Components: main",
            # unofficial mirror, FIXME: We'd rather merge the notalucid into here
            "Types: deb deb-src\n"
            "URIs: http://example.com/\n"
            "Suites: lucid lucid-updates\n"
            "Components: main",
        ]
        self.maxDiff = None
        self.assertEqual(expected, list(map(str, sources.list)))

    def test_deb822_explode(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "/dev/null")
        target = (
            apt_pkg.config.find_dir("dir::etc::sourceparts") + "enable_comps.sources"
        )
        line = "Types: deb\nURIs: http://archive.ubuntu.com/ubuntu\nSuites: lucid\nComponents: main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)

        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 1)
        self.assertIsInstance(
            sources.exploded_list()[0], aptsources.sourceslist.Deb822SourceEntry
        )

        sources.list[0].suites += ["fakesuite"]
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 2)
        self.assertIsInstance(
            sources.exploded_list()[0], aptsources.sourceslist.ExplodedDeb822SourceEntry
        )
        self.assertIsInstance(
            sources.exploded_list()[1], aptsources.sourceslist.ExplodedDeb822SourceEntry
        )
        self.assertEqual(sources.list[0].suites, ["lucid", "fakesuite"])
        sources.remove(sources.exploded_list()[1])
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 1)
        self.assertEqual(sources.list[0].suites, ["lucid"])

        sources.list[0].types += ["deb-src"]
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 2)
        self.assertIsInstance(
            sources.exploded_list()[0], aptsources.sourceslist.ExplodedDeb822SourceEntry
        )
        self.assertIsInstance(
            sources.exploded_list()[1], aptsources.sourceslist.ExplodedDeb822SourceEntry
        )
        self.assertEqual(sources.list[0].types, ["deb", "deb-src"])
        sources.remove(sources.exploded_list()[1])
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 1)
        self.assertEqual(sources.list[0].types, ["deb"])

        sources.list[0].uris += ["http://example.com"]
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 2)
        self.assertIsInstance(
            sources.exploded_list()[0], aptsources.sourceslist.ExplodedDeb822SourceEntry
        )
        self.assertIsInstance(
            sources.exploded_list()[1], aptsources.sourceslist.ExplodedDeb822SourceEntry
        )
        self.assertEqual(
            sources.list[0].uris,
            ["http://archive.ubuntu.com/ubuntu", "http://example.com"],
        )
        sources.remove(sources.exploded_list()[1])
        self.assertEqual(len(sources.list), 1)
        self.assertEqual(len(sources.exploded_list()), 1)
        self.assertEqual(sources.list[0].uris, ["http://archive.ubuntu.com/ubuntu"])

        # test setting attributes
        sources.list[0].uris += ["http://example.com"]
        with self.assertRaises(AttributeError):
            sources.exploded_list()[0].types = ["does not work"]
        with self.assertRaises(AttributeError):
            sources.exploded_list()[0].uris = ["does not work"]
        with self.assertRaises(AttributeError):
            sources.exploded_list()[0].suites = ["does not work"]
        with self.assertRaises(AttributeError):
            sources.exploded_list()[0].doesnotexist = ["does not work"]

        # test overriding
        sources.exploded_list()[0].type = "faketype"
        self.assertEqual(sources.list[0].type, "faketype")
        self.assertEqual(sources.list[1].type, "deb")
        sources.exploded_list()[0].type = "deb"
        self.assertEqual(sources.list[0].type, "deb")
        self.assertEqual(sources.list[1].type, "deb")
        sources.save()
        self.assertEqual(len(sources.list), 1)

    def testDistribution_short(self):
        """aptsources: Test distribution detection."""
        apt_pkg.config.set(
            "Dir::Etc::sourcelist", "data/aptsources/" "sources.list.testDistribution"
        )
        return self.commonTestDistribution()

    def testDistribution_deb822(self):
        """aptsources: Test distribution detection."""
        apt_pkg.config.set(
            "Dir::Etc::sourceparts",
            "data/aptsources/" "sources.list.d.testDistribution",
        )
        return self.commonTestDistribution()

    def commonTestDistribution(self):
        """aptsources: Test distribution detection."""
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        distro = aptsources.distro.get_distro(
            id="Ubuntu",
            codename="bionic",
            description="Ubuntu 18.04 LTS",
            release="18.04",
        )
        distro.get_sources(sources)
        # test if all suits of the current distro were detected correctly
        dist_templates = set()
        for s in sources:
            if s.template:
                dist_templates.add(s.template.name)
        # print dist_templates
        for d in (
            "hardy",
            "hardy-security",
            "hardy-updates",
            "intrepid",
            "hardy-backports",
        ):
            self.assertTrue(d in dist_templates)
        # test enable
        comp = "restricted"
        distro.enable_component(comp)
        found = {}
        for entry in sources:
            if (
                entry.type == "deb"
                and entry.uri == "http://de.archive.ubuntu.com/ubuntu/"
                and "edgy" in entry.dist
            ):
                for c in entry.comps:
                    if c == comp:
                        if entry.dist not in found:
                            found[entry.dist] = 0
                        found[entry.dist] += 1
        # print "".join([s.str() for s in sources])
        for key in found:
            self.assertEqual(found[key], 1)

        # add a not-already available component
        comp = "multiverse"
        distro.enable_component(comp)
        found = {}
        for entry in sources:
            if entry.type == "deb" and entry.template and entry.template.name == "edgy":
                for c in entry.comps:
                    if c == comp:
                        if entry.dist not in found.has_key:
                            found[entry.dist] = 0
                        found[entry.dist] += 1
        # print "".join([s.str() for s in sources])
        for key in found:
            self.assertEqual(found[key], 1)

    def test_enable_disabled_short(self):
        """LP: #1042916: Test enabling disabled entry."""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        return self.common_test_enable_disabled()

    def test_enable_disabled_deb822(self):
        """LP: #1042916: Test enabling disabled entry."""
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        return self.common_test_enable_disabled()

    def common_test_enable_disabled(self):
        """LP: #1042916: Test enabling disabled entry."""
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        disabled = sources.add(
            "deb",
            "http://fi.archive.ubuntu.com/ubuntu/",
            "precise",
            ["main"],
            file=sources.list[0].file,  # if we use deb822, enable deb822
        )
        disabled.set_enabled(False)
        enabled = sources.add(
            "deb",
            "http://fi.archive.ubuntu.com/ubuntu/",
            "precise",
            ["main"],
            file=sources.list[0].file,  # if we use deb822, enable deb822
        )
        self.assertEqual(disabled, enabled)
        self.assertFalse(disabled.disabled)

    def test_duplicate_uri_with_trailing_slash_short(self):
        """Test replacing entry with same uri except trailing slash"""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/" "sources.list")
        return self.common_test_duplicate_uri_with_trailing_slash()

    def test_duplicate_uri_with_trailing_slash_deb822(self):
        """Test replacing entry with same uri except trailing slash"""
        apt_pkg.config.set("Dir::Etc::sourceparts", "data/aptsources/" "sources.list.d")
        return self.common_test_duplicate_uri_with_trailing_slash()

    def common_test_duplicate_uri_with_trailing_slash(self):
        """Test replacing entry with same uri except trailing slash"""
        sources = aptsources.sourceslist.SourcesList(True, self.templates, deb822=True)
        line_wslash = "deb http://rslash.ubuntu.com/ubuntu/ precise main"
        line_woslash = "deb http://rslash.ubuntu.com/ubuntu precise main"
        entry_wslash = aptsources.sourceslist.SourceEntry(line_wslash)
        entry_woslash = aptsources.sourceslist.SourceEntry(line_woslash)
        self.assertEqual(entry_wslash, entry_woslash)
        count = len(sources.list)
        sourceslist_wslash = sources.add(
            entry_wslash.type, entry_wslash.uri, entry_wslash.dist, entry_wslash.comps
        )
        self.assertEqual(count + 1, len(sources.list))
        count = len(sources.list)
        sourceslist_woslash = sources.add(
            entry_woslash.type,
            entry_woslash.uri,
            entry_woslash.dist,
            entry_woslash.comps,
        )
        self.assertEqual(count, len(sources.list))
        self.assertEqual(sourceslist_wslash, sourceslist_woslash)

    def test_deb822_distro_enable_disable_component(self):
        """Test enabling and disabling a component in the distro sources.

        This ensures reasonable behavior when enabling and then disabling a component"""
        with tempfile.NamedTemporaryFile("w", suffix=".sources") as file:
            file.write(
                "# main archive\n"
                "Types: deb deb-src\n"
                "URIs: http://archive.ubuntu.com/ubuntu/\n"
                "Suites: noble noble-updates\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                "\n"
                "# security\n"
                "Types: deb deb-src\n"
                "URIs: http://security.ubuntu.com/ubuntu/\n"
                "Suites: noble-security\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
            )
            file.flush()

            apt_pkg.config.set("Dir::Etc::sourcelist", file.name)
            sources = aptsources.sourceslist.SourcesList(
                True, self.templates, deb822=True
            )
            distro = aptsources.distro.get_distro(
                id="Ubuntu",
                codename="noble",
                description="Ubuntu 24.04 LTS",
                release="24.04",
            )

            self.assertEqual(len(sources.list), 2)
            distro.get_sources(sources)
            distro.enable_component("multiverse")
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe multiverse\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe multiverse\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            # Disable it again
            # FIXME: The child entries will no longer be valid at this point, so we have to call
            #        get_sources(), but it's not clear whether this should be considered a bug -
            #        there may have been other changes rendering entries no longer valid that distro
            #        is holding on to.
            distro.get_sources(sources)
            distro.disable_component("multiverse")
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            # Disable universe as well
            distro.get_sources(sources)
            distro.disable_component("universe")
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

    def test_deb822_distro_enable_disable_component_mixed_origin(self):
        """Test enabling and disabling a component in the distro sources, with mixed origin

        Here we ensure that we still get idempotent behavior of disable after enable even if the
        entry we were modifying also had a non-official repository in it."""
        with tempfile.NamedTemporaryFile("w", suffix=".sources") as file:
            file.write(
                "# main archive\n"
                "Types: deb deb-src\n"
                "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                "Suites: noble noble-updates\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                "\n"
                "# security\n"
                "Types: deb deb-src\n"
                "URIs: http://security.ubuntu.com/ubuntu/\n"
                "Suites: noble-security\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
            )
            file.flush()

            apt_pkg.config.set("Dir::Etc::sourcelist", file.name)
            sources = aptsources.sourceslist.SourcesList(
                True, self.templates, deb822=True
            )
            distro = aptsources.distro.get_distro(
                id="Ubuntu",
                codename="noble",
                description="Ubuntu 24.04 LTS",
                release="24.04",
            )

            self.assertEqual(len(sources.list), 2)
            distro.get_sources(sources)
            distro.enable_component("multiverse")
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe multiverse\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe multiverse\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            # Disable it again
            # FIXME: The child entries will no longer be valid at this point, so we have to call
            #        get_sources(), but it's not clear whether this should be considered a bug -
            #        there may have been other changes rendering entries no longer valid that distro
            #        is holding on to.
            distro.get_sources(sources)
            distro.disable_component("multiverse")
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            # Disable universe too. The behaviour here is interesting: Distro only disables
            # universe for the official source, so we end up with the non-official source split out.
            distro.get_sources(sources)
            distro.disable_component("universe")
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# main archive\n"  # note it keeps the comment on the split out child
                    "Types: deb deb-src\n"
                    "URIs: http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

    def test_deb822_distro_enable_disable_child_source_mixed_origins(self):
        """Test enabling and disabling a child source (proposed) in the distro sources, with mixed origin

        Here we ensure that we still get idempotent behavior of disable after enable even if the
        entry we were modifying also had a non-official repository in it."""
        with tempfile.NamedTemporaryFile("w", suffix=".sources") as file:
            file.write(
                "# main archive\n"
                "Types: deb deb-src\n"
                "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                "Suites: noble noble-updates\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                "\n"
                "# security\n"
                "Types: deb deb-src\n"
                "URIs: http://security.ubuntu.com/ubuntu/\n"
                "Suites: noble-security\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
            )
            file.flush()

            apt_pkg.config.set("Dir::Etc::sourcelist", file.name)
            sources = aptsources.sourceslist.SourcesList(
                True, self.templates, deb822=True
            )
            distro = aptsources.distro.get_distro(
                id="Ubuntu",
                codename="noble",
                description="Ubuntu 24.04 LTS",
                release="24.04",
            )

            self.assertEqual(len(sources.list), 2)
            distro.get_sources(sources)
            distro.get_source_code = True
            distro.add_source(dist="noble-proposed")

            # FIXME: Component ordering is not stable right now
            for entry in sources.list:
                entry.comps = sorted(entry.comps)
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None

                # FIXME: In an optimal world it would look like this
                self.assertNotEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates noble-proposed\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            with open(file.name) as readonly:
                self.maxDiff = None
                # Sadly our merge algorithm does not always produce optimal merges, because it merges the unofficial entry first
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble-proposed\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            # Disable it again
            # FIXME: The child entries will no longer be valid at this point, so we have to call
            #        get_sources(), but it's not clear whether this should be considered a bug -
            #        there may have been other changes rendering entries no longer valid that distro
            #        is holding on to.
            distro.get_sources(sources)
            for child in distro.child_sources + distro.source_code_sources:
                if child.dist.endswith("proposed"):
                    sources.remove(child)
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb deb-src\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb deb-src\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

    def test_deb822_distro_enable_disable_child_source_mixed_origins_no_source_code(
        self,
    ):
        """Test enabling and disabling a child source (proposed) in the distro sources, with mixed origin

        Here we ensure that we still get idempotent behavior of disable after enable even if the
        entry we were modifying also had a non-official repository in it."""
        with tempfile.NamedTemporaryFile("w", suffix=".sources") as file:
            file.write(
                "# main archive\n"
                "Types: deb\n"
                "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                "Suites: noble noble-updates\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                "\n"
                "# security\n"
                "Types: deb\n"
                "URIs: http://security.ubuntu.com/ubuntu/\n"
                "Suites: noble-security\n"
                "Components: main universe\n"
                "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
            )
            file.flush()

            apt_pkg.config.set("Dir::Etc::sourcelist", file.name)
            sources = aptsources.sourceslist.SourcesList(
                True, self.templates, deb822=True
            )
            distro = aptsources.distro.get_distro(
                id="Ubuntu",
                codename="noble",
                description="Ubuntu 24.04 LTS",
                release="24.04",
            )

            self.assertEqual(len(sources.list), 2)
            distro.get_sources(sources)
            self.assertEqual(
                [(s.type, s.uri, s.dist, s.comps) for s in distro.main_sources]
                + ["separator"]
                + [(s.type, s.uri, s.dist, s.comps) for s in distro.child_sources],
                [
                    (
                        "deb",
                        "http://archive.ubuntu.com/ubuntu/",
                        "noble",
                        ["main", "universe"],
                    ),
                    "separator",
                    (
                        "deb",
                        "http://archive.ubuntu.com/ubuntu/",
                        "noble-updates",
                        ["main", "universe"],
                    ),
                    (
                        "deb",
                        "http://security.ubuntu.com/ubuntu/",
                        "noble-security",
                        ["main", "universe"],
                    ),
                ],
            )
            distro.add_source(dist="noble-proposed")

            # FIXME: Component ordering is not stable right now
            for entry in sources.list:
                entry.comps = sorted(entry.comps)
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None

                # FIXME: In an optimal world it would look like this
                self.assertNotEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble noble-updates noble-proposed\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# main archive\n"
                    "Types: deb\n"
                    "URIs: http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            with open(file.name) as readonly:
                self.maxDiff = None
                # Sadly our merge algorithm does not always produce optimal merges, because it merges the unofficial entry first
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "Types: deb\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/\n"
                    "Suites: noble-proposed\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )

            # Disable it again
            # FIXME: The child entries will no longer be valid at this point, so we have to call
            #        get_sources(), but it's not clear whether this should be considered a bug -
            #        there may have been other changes rendering entries no longer valid that distro
            #        is holding on to.
            distro.get_sources(sources)
            for child in distro.child_sources + distro.source_code_sources:
                if child.dist.endswith("proposed"):
                    sources.remove(child)
            sources.save()

            with open(file.name) as readonly:
                self.maxDiff = None
                self.assertEqual(
                    readonly.read(),
                    "# main archive\n"
                    "Types: deb\n"
                    "URIs: http://archive.ubuntu.com/ubuntu/ http://unofficial.example.com/\n"
                    "Suites: noble noble-updates\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n"
                    "\n"
                    "# security\n"
                    "Types: deb\n"
                    "URIs: http://security.ubuntu.com/ubuntu/\n"
                    "Suites: noble-security\n"
                    "Components: main universe\n"
                    "Signed-By: /usr/share/keyrings/ubuntu-archive-keyrings.gpg\n",
                )


if __name__ == "__main__":
    os.chdir(os.path.dirname(__file__))
    unittest.main()



#!/usr/bin/python3
import os.path

import apt_pkg

import apt

if __name__ == "__main__":
    apt_pkg.config.set("APT::Update::Pre-Invoke::", "touch /tmp/update-about-to-run")
    apt_pkg.config.set("APT::Update::Post-Invoke::", "touch /tmp/update-was-run")
    c = apt.Cache()
    res = c.update(apt.progress.TextFetchProgress())
    print("res: ", res)
    assert os.path.exists("/tmp/update-about-to-run")



#!/usr/bin/env python2.4
#
# Test for the pkgCache code
#

import os
import sys

import apt_pkg

if __name__ == "__main__":
    lock = "/tmp/test.lck"

    apt_pkg.init()

    # system-lock
    apt_pkg.pkgsystem_lock()

    pid = os.fork()
    if pid == 0:
        try:
            apt_pkg.pkgsystem_lock()
        except SystemError as s:
            print("Can't get lock: (error text:\n%s)" % s)
        sys.exit(0)

    apt_pkg.pkgsystem_unlock()

    # low-level lock
    fd = apt_pkg.get_lock(lock, True)
    print("Lockfile fd: %s" % fd)

    # try to get lock without error flag
    pid = os.fork()
    if pid == 0:
        # child
        fd = apt_pkg.get_lock(lock, False)
        print("Lockfile fd (child): %s" % fd)
        sys.exit(0)

    # try to get lock with error flag
    pid = os.fork()
    if pid == 0:
        # child
        fd = apt_pkg.get_lock(lock, True)
        print("Lockfile fd (child): %s" % fd)
        sys.exit(0)



#!/usr/bin/python3

import apt_pkg

apt_pkg.init()

# cache = apt_pkg.Cache()
# sources = apt_pkg.SourceRecords(cache)

sources = apt_pkg.SourceRecords()
sources.restart()
while sources.lookup("hello"):
    print(
        sources.package,
        sources.version,
        sources.maintainer,
        sources.section,
        repr(sources.binaries),
    )
    print(sources.files)
    print(sources.index.archive_uri(sources.files[0][2]))



#!/usr/bin/python3
#
# Copyright (C) 2010 Michael Vogt <mvo@ubuntu.com>
# Copyright (C) 2012 Canonical Ltd.
# Author: Colin Watson <cjwatson@ubuntu.com>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Unit tests for verifying the correctness of apt_pkg.TagFile"""

import glob
import os
import shutil
import sys
import tempfile
import unittest

from test_all import get_library_dir

libdir = get_library_dir()
if libdir:
    sys.path.insert(0, libdir)

import apt_pkg
import testcommon


class TestOpenMaybeClearSigned(testcommon.TestCase):
    def test_open_trivial(self):
        basepath = os.path.dirname(__file__)
        fd = apt_pkg.open_maybe_clear_signed_file(
            os.path.join(basepath, "./data/test_debs/hello_2.5-1.dsc")
        )
        with os.fdopen(fd) as f:
            data = f.read()
        self.assertTrue(data.startswith("Format: 1.0\n"))

    def test_open_normal(self):
        basepath = os.path.dirname(__file__)
        fd = apt_pkg.open_maybe_clear_signed_file(
            os.path.join(basepath, "./data/misc/foo_Release")
        )
        with os.fdopen(fd) as f:
            data = f.read()
        self.assertTrue(data.startswith("Origin: Ubuntu\n"))

    def xtest_open_does_not_exit(self):
        with self.assertRaises(SystemError):
            apt_pkg.open_maybe_clear_signed_file("does-not-exists")


class TestTagFile(testcommon.TestCase):
    """test the apt_pkg.TagFile"""

    def setUp(self):
        testcommon.TestCase.setUp(self)
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_tag_file(self):
        basepath = os.path.dirname(__file__)
        tagfilepath = os.path.join(basepath, "./data/tagfile/*")
        # test once for compressed and uncompressed
        for testfile in glob.glob(tagfilepath):
            # test once using the open() method and once using the path
            tagfile = apt_pkg.TagFile(testfile)
            for i, stanza in enumerate(tagfile):
                pass
            self.assertEqual(i, 2)
            with open(testfile) as f:
                tagfile = apt_pkg.TagFile(f)
                for i, stanza in enumerate(tagfile):
                    pass
                self.assertEqual(i, 2)

    def test_errors(self):
        # Raises SystemError via lbiapt
        self.assertRaises(SystemError, apt_pkg.TagFile, "not-there-no-no")
        # Raises Type error
        self.assertRaises(TypeError, apt_pkg.TagFile, object())

    def test_utf8(self):
        value = "Tést Persön <test@example.org>"
        packages = os.path.join(self.temp_dir, "Packages")
        with open(packages, "w", encoding="UTF-8") as packages_file:
            print("Maintainer: %s" % value, file=packages_file)
            print("", file=packages_file)
        with open(packages, encoding="UTF-8") as packages_file:
            tagfile = apt_pkg.TagFile(packages_file)
            tagfile.step()
            self.assertEqual(value, tagfile.section["Maintainer"])

    def test_latin1(self):
        value = "Tést Persön <test@example.org>"
        packages = os.path.join(self.temp_dir, "Packages")
        with open(packages, "w", encoding="ISO-8859-1") as packages_file:
            print("Maintainer: %s" % value, file=packages_file)
            print("", file=packages_file)
        with open(packages) as packages_file:
            tagfile = apt_pkg.TagFile(packages_file, bytes=True)
            tagfile.step()
            self.assertEqual(value.encode("ISO-8859-1"), tagfile.section["Maintainer"])
        with open(packages, encoding="ISO-8859-1") as packages_file:
            tagfile = apt_pkg.TagFile(packages_file)
            tagfile.step()
            self.assertEqual(value, tagfile.section["Maintainer"])

    def test_mixed(self):
        value = "Tést Persön <test@example.org>"
        packages = os.path.join(self.temp_dir, "Packages")
        with open(packages, "w", encoding="UTF-8") as packages_file:
            print("Maintainer: %s" % value, file=packages_file)
            print("", file=packages_file)
        with open(packages, "a", encoding="ISO-8859-1") as packages_file:
            print("Maintainer: %s" % value, file=packages_file)
            print("", file=packages_file)
        with open(packages) as packages_file:
            tagfile = apt_pkg.TagFile(packages_file, bytes=True)
            tagfile.step()
            self.assertEqual(value.encode("UTF-8"), tagfile.section["Maintainer"])
            tagfile.step()
            self.assertEqual(value.encode("ISO-8859-1"), tagfile.section["Maintainer"])


class TestTagSection(testcommon.TestCase):
    """test the apt_pkg.TagFile"""

    def setUp(self):
        testcommon.TestCase.setUp(self)
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_write(self):
        ts = apt_pkg.TagSection("a: 1\nb: 2\nc: 3\n")
        outpath = os.path.join(self.temp_dir, "test")
        with open(outpath, "w") as outfile:
            ts.write(outfile, [], [])
        with open(outpath) as outfile:
            self.assertEqual(outfile.read(), "a: 1\nb: 2\nc: 3\n")

    def test_write_order(self):
        ts = apt_pkg.TagSection("a: 1\nb: 2\nc: 3\n")
        outpath = os.path.join(self.temp_dir, "test")
        with open(outpath, "w") as outfile:
            ts.write(outfile, ["a", "c", "b"], [])
        with open(outpath) as outfile:
            self.assertEqual(outfile.read(), "a: 1\nc: 3\nb: 2\n")

    def test_write_invalid_order(self):
        ts = apt_pkg.TagSection("a: 1\nb: 2\nc: 3\n")
        outpath = os.path.join(self.temp_dir, "test")
        with open(outpath, "w") as outfile:
            self.assertRaises(TypeError, ts.write, outfile, ["a", 1, "b"], [])

    def test_write_remove(self):
        ts = apt_pkg.TagSection("a: 1\nb: 2\nc: 3\n")
        outpath = os.path.join(self.temp_dir, "test")
        with open(outpath, "w") as outfile:
            ts.write(outfile, ["a", "c", "b"], [apt_pkg.TagRemove("a")])
        with open(outpath) as outfile:
            self.assertEqual(outfile.read(), "c: 3\nb: 2\n")

    def test_write_rewrite(self):
        ts = apt_pkg.TagSection("a: 1\nb: 2\nc: 3\n")
        outpath = os.path.join(self.temp_dir, "test")
        with open(outpath, "w") as outfile:
            ts.write(outfile, ["a", "c", "b"], [apt_pkg.TagRewrite("a", "AA")])
        with open(outpath) as outfile:
            self.assertEqual(outfile.read(), "a: AA\nc: 3\nb: 2\n")

    def test_write_rename(self):
        ts = apt_pkg.TagSection("a: 1\nb: 2\nc: 3\n")
        outpath = os.path.join(self.temp_dir, "test")
        with open(outpath, "w") as outfile:
            ts.write(outfile, ["a", "z", "b"], [apt_pkg.TagRename("c", "z")])
        with open(outpath) as outfile:
            self.assertEqual(outfile.read(), "a: 1\nz: 3\nb: 2\n")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
import sys

import apt


def dependencies(cache, pkg, deps, key="Depends"):
    # print "pkg: %s (%s)" % (pkg.name, deps)
    candver = cache._depcache.get_candidate_ver(pkg._pkg)
    if candver is None:
        return deps
    dependslist = candver.depends_list
    if key in dependslist:
        for depVerList in dependslist[key]:
            for dep in depVerList:
                if dep.target_pkg.name in cache:
                    if (
                        pkg.name != dep.target_pkg.name
                        and dep.target_pkg.name not in deps
                    ):
                        deps.add(dep.target_pkg.name)
                        dependencies(cache, cache[dep.target_pkg.name], deps, key)
    return deps


pkgname = sys.argv[1]
c = apt.Cache()
pkg = c[pkgname]

deps = set()

deps = dependencies(c, pkg, deps, "Depends")
print(" ".join(deps))

preDeps = set()
preDeps = dependencies(c, pkg, preDeps, "PreDepends")
print(" ".join(preDeps))



#!/usr/bin/python3

import gc
import time

import apt_pkg

import apt

cache = apt.Cache()

# memleak
for i in range(100):
    cache.open(None)
    print(cache["apt"].name)
    time.sleep(1)
    gc.collect()
    f = open("%s" % i, "w")
    for obj in gc.get_objects():
        f.write("%s\n" % str(obj))
    f.close()

# memleak
# for i in range(100):
#       cache = apt.Cache()
#       time.sleep(1)
#       cache = None
#       gc.collect()

# no memleak, but more or less the apt.Cache.open() code
for i in range(100):
    cache = apt_pkg.Cache()
    depcache = apt_pkg.DepCache(cache)
    records = apt_pkg.PackageRecords(cache)
    list = apt_pkg.SourceList()
    list.read_main_list()
    dict = {}
    for pkg in cache.packages:
        if len(pkg.version_list) > 0:
            dict[pkg.name] = apt.package(cache, depcache, records, list, None, pkg)

    print(cache["apt"])
    time.sleep(1)

    gc.collect()



import warnings

warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
import apt

if __name__ == "__main__":
    progress = apt.progress.OpTextProgress()
    cache = apt.Cache(progress)
    print(cache)
    for pkg in cache:
        if pkg.is_upgradable:
            pkg.mark_install()
    for pkg in cache.get_changes():
        # print pkg.name()
        pass
    print("Broken: %s " % cache._depcache.broken_count)
    print("inst_count: %s " % cache._depcache.inst_count)

    # get a new cache
    cache = apt.Cache(progress)
    for name in cache.keys():
        import random

        if random.randint(0, 1) == 1:
            cache[name].mark_delete()
    print("Broken: %s " % cache._depcache.broken_count)
    print("del_count: %s " % cache._depcache.del_count)



#!/usr/bin/python3
#
# Copyright (C) 2010 Julian Andres Klode <jak@debian.org>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Unit tests for verifying the correctness of check_dep, etc in apt_pkg."""
import itertools
import unittest

import apt_pkg
import testcommon

import apt.package


class TestDependencies(testcommon.TestCase):
    def testCheckDep(self):
        """dependencies: Test apt_pkg.CheckDep() for '<' and '>'

        The CheckDep function should treat '<' as '<=' and '>' as '>=', for
        compatibility reasons."""
        if not hasattr(apt_pkg, "CheckDep"):
            return
        self.assertFalse(apt_pkg.CheckDep("1", "<", "0"))
        self.assertTrue(apt_pkg.CheckDep("1", "<", "1"))
        self.assertTrue(apt_pkg.CheckDep("1", "<", "2"))

        self.assertFalse(apt_pkg.CheckDep("0", ">", "1"))
        self.assertTrue(apt_pkg.CheckDep("1", ">", "1"))
        self.assertTrue(apt_pkg.CheckDep("2", ">", "1"))

    def test_check_dep(self):
        "dependencies: Test apt_pkg.check_dep()"
        self.assertFalse(apt_pkg.check_dep("1", "<<", "0"))
        self.assertFalse(apt_pkg.check_dep("1", "<<", "1"))
        self.assertTrue(apt_pkg.check_dep("1", "<<", "2"))

        self.assertFalse(apt_pkg.check_dep("1", "<", "0"))
        self.assertFalse(apt_pkg.check_dep("1", "<", "1"))
        self.assertTrue(apt_pkg.check_dep("1", "<", "2"))

        self.assertFalse(apt_pkg.check_dep("1", "<=", "0"))
        self.assertTrue(apt_pkg.check_dep("1", "<=", "1"))
        self.assertTrue(apt_pkg.check_dep("1", "<=", "2"))

        self.assertFalse(apt_pkg.check_dep("0", "=", "1"))
        self.assertTrue(apt_pkg.check_dep("1", "=", "1"))
        self.assertFalse(apt_pkg.check_dep("2", "=", "1"))

        self.assertFalse(apt_pkg.check_dep("0", ">=", "1"))
        self.assertTrue(apt_pkg.check_dep("1", ">=", "1"))
        self.assertTrue(apt_pkg.check_dep("2", ">=", "1"))

        self.assertFalse(apt_pkg.check_dep("0", ">", "1"))
        self.assertFalse(apt_pkg.check_dep("1", ">", "1"))
        self.assertTrue(apt_pkg.check_dep("2", ">", "1"))

        self.assertFalse(apt_pkg.check_dep("0", ">>", "1"))
        self.assertFalse(apt_pkg.check_dep("1", ">>", "1"))
        self.assertTrue(apt_pkg.check_dep("2", ">>", "1"))

    def test_parse_depends_multiarch(self):
        # strip multiarch
        deps = apt_pkg.parse_depends("po4a:native", True)
        self.assertEqual(deps[0][0][0], "po4a")
        # do not strip multiarch
        deps = apt_pkg.parse_depends("po4a:native", False)
        self.assertEqual(deps[0][0][0], "po4a:native")

    def test_parse_depends(self):
        """dependencies: Test apt_pkg.parse_depends()"""
        deps = apt_pkg.parse_depends("p1a (<< 1a) | p1b (>> 1b)")
        self.assertTrue(isinstance(deps, list))
        self.assertEqual(len(deps), 1)
        self.assertTrue(isinstance(deps[0], list))
        self.assertEqual(len(deps[0]), 2)
        self.assertEqual(len(deps[0][0]), 3)
        self.assertEqual(len(deps[0][1]), 3)
        self.assertEqual(deps[0][0][0], "p1a")
        self.assertEqual(deps[0][0][1], "1a")
        self.assertEqual(deps[0][0][2], "<")
        self.assertEqual(deps[0][1][0], "p1b")
        self.assertEqual(deps[0][1][1], "1b")
        self.assertEqual(deps[0][1][2], ">")

        # Check that the type of comparison is parsed correctly.
        self.assertEqual("<", apt_pkg.parse_depends("p1 (<< 1)")[0][0][2])
        self.assertEqual("<=", apt_pkg.parse_depends("p1 (< 1)")[0][0][2])
        self.assertEqual("<=", apt_pkg.parse_depends("p1 (<= 1)")[0][0][2])
        self.assertEqual("=", apt_pkg.parse_depends("p1 (= 1)")[0][0][2])
        self.assertEqual(">=", apt_pkg.parse_depends("p1 (>= 1)")[0][0][2])
        self.assertEqual(">=", apt_pkg.parse_depends("p1 (> 1)")[0][0][2])
        self.assertEqual(">", apt_pkg.parse_depends("p1 (>> 1)")[0][0][2])

    def test_parse_src_depends(self):
        """dependencies: Test apt_pkg.parse_src_depends()."""
        # Check that architecture exclusion works
        # depends_this: Current architecture is included
        # depends_this_too: Another architecture is excluded
        # depends_other: The current architecture is excluded
        # depends_other: Another architecture is requested.
        architecture = apt_pkg.config["APT::Architecture"]
        depends_this = apt_pkg.parse_src_depends("p [%s]" % architecture)
        depends_this_too = apt_pkg.parse_src_depends("p [!not-existing-arch]")
        depends_other = apt_pkg.parse_src_depends("p [!%s]" % architecture)
        depends_other_too = apt_pkg.parse_src_depends("p [not-existing-arch]")

        self.assertEqual(len(depends_this), len(depends_this_too), 1)
        self.assertEqual(len(depends_other), len(depends_other_too), 0)

    def test_dstr(self):
        """Test apt.package.BaseDependency.__dstr"""
        dstr = apt.package.BaseDependency._BaseDependency__dstr
        equal = {"<": {"<<", "<"}, "=": {"==", "="}, ">": {">>", ">"}}
        operators = ["<<", "<", "<=", "!=", "=", "==", ">=", ">", ">>"]

        for a, b in itertools.product(equal.keys(), operators):
            if b in equal[a]:
                self.assertEqual(dstr(a), b)
                self.assertEqual(b, dstr(a))
            else:
                self.assertNotEqual(dstr(a), b)
                self.assertNotEqual(b, dstr(a))

    def testParseDepends(self):
        """dependencies: Test apt_pkg.ParseDepends()."""
        if not hasattr(apt_pkg, "ParseDepends"):
            return
        # Check that the type of comparison is parsed correctly.
        self.assertEqual("<<", apt_pkg.ParseDepends("p1 (<< 1)")[0][0][2])
        self.assertEqual("<=", apt_pkg.ParseDepends("p1 (< 1)")[0][0][2])
        self.assertEqual("<=", apt_pkg.ParseDepends("p1 (<= 1)")[0][0][2])
        self.assertEqual("=", apt_pkg.ParseDepends("p1 (= 1)")[0][0][2])
        self.assertEqual(">=", apt_pkg.ParseDepends("p1 (>= 1)")[0][0][2])
        self.assertEqual(">=", apt_pkg.ParseDepends("p1 (> 1)")[0][0][2])
        self.assertEqual(">>", apt_pkg.ParseDepends("p1 (>> 1)")[0][0][2])


if __name__ == "__main__":
    unittest.main()



import apt_pkg


def main():
    apt_pkg.init_config()

    print("Native architecture:", apt_pkg.config["APT::Architecture"])
    print("All architectures:", apt_pkg.config.value_list("APT::Architectures"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
#  get_ubuntu_lp_mirrors.py
#
#  Download the latest list with available Ubuntu mirrors from Launchpad.net
#  and extract the hosts from the raw page
#
#  Copyright (c) 2006 Free Software Foundation Europe
#
#  Author: Sebastian Heinlein <glatzor@ubuntu.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import sys

import feedparser

d = feedparser.parse("https://launchpad.net/ubuntu/+archivemirrors-rss")
# d = feedparser.parse(open("+archivemirrors-rss"))

countries = {}

for entry in d.entries:
    countrycode = entry.mirror_countrycode
    if countrycode not in countries:
        countries[countrycode] = set()
    for link in entry.links:
        countries[countrycode].add(link.href)


keys = sorted(countries)

if len(keys) == 0:
    sys.stderr.write(
        "E: Could not read the mirror list due to some issue"
        " -- status code: %s\n" % d.status
    )
    sys.exit(1)

print("mirror://mirrors.ubuntu.com/mirrors.txt")
for country in keys:
    print("#LOC:%s" % country)
    print("\n".join(sorted(countries[country])))



#!/usr/bin/python3
# example how to deal with the depcache

import sys

import apt
from apt.progress import InstallProgress


class TextInstallProgress(InstallProgress):
    def __init__(self):
        apt.progress.InstallProgress.__init__(self)
        self.last = 0.0

    def updateInterface(self):
        InstallProgress.updateInterface(self)
        if self.last >= self.percent:
            return
        sys.stdout.write(f"\r[{self.percent}] {self.status}\n")
        sys.stdout.flush()
        self.last = self.percent

    def conffile(self, current, new):
        print(f"conffile prompt: {current} {new}")

    def error(self, errorstr):
        print("got dpkg error: '%s'" % errorstr)


cache = apt.Cache(apt.progress.OpTextProgress())

fprogress = apt.progress.TextFetchProgress()
iprogress = TextInstallProgress()

pkg = cache["3dchess"]

# install or remove, the importend thing is to keep us busy :)
if pkg.is_installed:
    print("Going to delete %s" % pkg.name)
    pkg.mark_delete()
else:
    print("Going to install %s" % pkg.name)
    pkg.mark_install()
res = cache.commit(fprogress, iprogress)
print(res)

sys.exit(0)



#!/usr/bin/python3
#
# Copyright (C) 2018 Michael Vogt <mvo@ubuntu.com>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Unit tests for verifying the correctness of source records in apt_pkg."""

import os
import shutil
import sys
import unittest

from test_all import get_library_dir

libdir = get_library_dir()
if libdir:
    sys.path.insert(0, libdir)

import apt_pkg
import testcommon

import apt


class TestSourceRecords(testcommon.TestCase):
    def setUp(self):
        testcommon.TestCase.setUp(self)

        rootdir = "./data/tmp"
        if os.path.exists(rootdir):
            shutil.rmtree(rootdir)
        try:
            os.makedirs(os.path.join(rootdir, "etc", "apt", "apt.conf.d"))
        except OSError:
            pass

        for k in apt_pkg.config.keys():
            apt_pkg.config.clear(k)

        apt_pkg.config["Dir"] = os.path.abspath(rootdir)
        apt_pkg.init_config()

        # set a local sources.list that does not need the network
        base_sources = os.path.abspath(
            os.path.join(rootdir, "etc", "apt", "sources.list")
        )
        # main sources.list
        sources_list = base_sources
        with open(sources_list, "w") as f:
            repo = os.path.abspath("./data/test-source-repo")
            f.write("deb-src [trusted=yes] copy:%s /\n" % repo)

        self.assertTrue(os.path.exists(sources_list))

        # update a single sources.list
        cache = apt.Cache(rootdir=rootdir)
        cache.update(sources_list=sources_list)

    def test_source_records_smoke(self):
        src = apt_pkg.SourceRecords()
        self.assertTrue(src.step())

        self.assertEqual(src.maintainer, "Julian Andres Klode <jak@debian.org>")  # noqa
        self.assertEqual(src.binaries, ["dh-autoreconf"])
        self.assertEqual(src.package, "dh-autoreconf")

        self.assertEqual(2, len(src.files))

        # unpacking as a tuple works as before
        md5, size, path, type_ = f = src.files[0]
        self.assertEqual(md5, None)
        self.assertEqual(size, 1578)
        self.assertEqual(path, "dh-autoreconf_16.dsc")
        self.assertEqual(type_, "dsc")
        # access using getters
        self.assertTrue(isinstance(f.hashes, apt_pkg.HashStringList))
        self.assertEqual(
            str(f.hashes[0]),
            "SHA512:4b1a3299f2a8b01b0c75db97fd16cb39919949c74d19ea6cf28e1bbd4891d3515b3e2b90b96a64df665cebf6d95409e704e670909ae91fcfe92409ee1339bffc",
        )  # noqa
        self.assertEqual(str(f.hashes[1]), "Checksum-FileSize:1578")
        self.assertEqual(
            str(f.hashes[2]),
            "SHA256:1c1b2ab5f1ae5496bd50dbb3c30e9b7d181a06c8d02ee8d7e9c35ed6f2a69b5f",
        )  # noqa
        self.assertEqual(
            str(f.hashes[3]), "SHA1:c9bf7a920013021dad5fbd898dfd5a79c7a150f9"
        )  # noqa
        self.assertEqual(
            str(f.hashes[4]), "MD5Sum:6576a28fe1918ce10bd31543ba545901"
        )  # noqa
        self.assertEqual(f.size, 1578)
        self.assertEqual(f.path, "dh-autoreconf_16.dsc")
        self.assertEqual(f.type, "dsc")

        # unpacking as a tuple works as before
        md5, size, path, type_ = f = src.files[1]
        self.assertEqual(md5, None)
        self.assertEqual(size, 7372)
        self.assertEqual(path, "dh-autoreconf_16.tar.xz")
        self.assertEqual(type_, "tar")
        # access using getters
        self.assertTrue(isinstance(f.hashes, apt_pkg.HashStringList))
        self.assertEqual(
            str(f.hashes[0]),
            "SHA512:10448dd179ec12bf4310a9a514110a85f56e51893aa36a97ac3a6f8d7ce99d099e62cfdb78e271e2d94431e8832da0f643de821b6643b80e3f0b0f5d682cf9a9",
        )  # noqa
        self.assertEqual(str(f.hashes[1]), "Checksum-FileSize:7372")  # noqa
        self.assertEqual(
            str(f.hashes[2]),
            "SHA256:5c6a6a362907327bec77a867ff3fd0eceba8015d1b881b48275aff7e4ce0f629",
        )  # noqa
        self.assertEqual(
            str(f.hashes[3]), "SHA1:58459600164398ad6807ddd877a6f814c799c62c"
        )  # noqa
        self.assertEqual(
            str(f.hashes[4]), "MD5Sum:302c8bf43db02412e3f2197fd0f2ee0f"
        )  # noqa
        self.assertEqual(f.size, 7372)
        self.assertEqual(f.path, "dh-autoreconf_16.tar.xz")
        self.assertEqual(f.type, "tar")

        self.assertFalse(src.step())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
"""Check the archive for missing dependencies"""
import apt_pkg


def fmt_dep(dep):
    """Format a Dependency object [of apt_pkg] as a string."""
    ret = dep.target_pkg.name
    if dep.target_ver:
        ret += f" ({dep.comp_type} {dep.target_ver})"
    return ret


def check_version(pkgver):
    """Check the version of the package"""
    missing = []

    for or_group in pkgver.depends_list.get(
        "Pre-Depends", []
    ) + pkgver.depends_list.get("Depends", []):
        if not any(dep.all_targets() for dep in or_group):
            # If none of the or-choices can be satisfied, add it to missing
            missing.append(or_group)

    if missing:
        print("Package:", pkgver.parent_pkg.name)
        print("Version:", pkgver.ver_str)
        print("Missing:")
        print(
            ", ".join(" | ".join(fmt_dep(dep) for dep in or_group))
            for or_group in missing
        )
        print()


def main():
    """The main function."""
    apt_pkg.init_config()
    apt_pkg.init_system()

    cache = apt_pkg.Cache()

    for pkg in sorted(cache.packages, key=lambda pkg: pkg.name):
        # pkg is from a list of packages, sorted by name.
        for version in pkg.version_list:
            # Check every version
            for pfile, _ in version.file_list:
                if (
                    pfile.origin == "Debian"
                    and pfile.component == "main"
                    and pfile.archive == "unstable"
                ):
                    # We only want packages from Debian unstable main.
                    check_version(version)
                    break


if __name__ == "__main__":
    main()



#!/usr/bin/python3
#
# Copyright (C) 2010 Michael Vogt <mvo@ubuntu.com>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Unit tests for verifying the correctness of apt.progress"""
import os
import unittest

import apt_pkg
import testcommon

import apt


class TestAcquireProgress(apt.progress.base.AcquireProgress):
    def pulse(self, owner):
        self.pulsed = True
        # there should be a return value here, either (True,False)
        # but often this is forgoten (and causes odd error messages)
        # so the lib supports it. we test the lack of support value here


class TestProgress(testcommon.TestCase):
    def setUp(self):
        testcommon.TestCase.setUp(self)
        basedir = os.path.abspath(os.path.dirname(__file__))
        # setup apt_pkg config
        apt_pkg.init()
        apt_pkg.config.set("APT::Architecture", "amd64")
        apt_pkg.config.set("Dir::Etc", basedir)
        # TODO: /dev/null is not a dir, perhaps find something better
        apt_pkg.config.set("Dir::Etc::sourceparts", "/dev/null")
        apt_pkg.config.set("Dir::Etc::preferencesparts", "/dev/null")
        # setup lists dir
        if not os.path.exists("./tmp/partial"):
            os.makedirs("./tmp/partial")
        apt_pkg.config.set("Dir::state::lists", "./tmp")
        # create artifical line
        deb_line = "deb [allow-insecure=yes] file:%s/data/fake-packages/ /\n" % basedir
        with open("fetch_sources.list", "w") as fobj:
            fobj.write(deb_line)
        apt_pkg.config.set("Dir::Etc::sourcelist", "fetch_sources.list")
        apt_pkg.config.clear("APT::Update::Post-Invoke")
        apt_pkg.config.clear("APT::Update::Post-Invoke-Success")

    def test_acquire_progress(self):
        progress = TestAcquireProgress()
        cache = apt.Cache()
        cache.update(progress)
        self.assertTrue(progress.pulsed)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
"""Print out the URIs of all indexes files.

This behaves somewhat like apt-get --print-uris update."""
import apt_pkg


def main():
    apt_pkg.init_config()
    apt_pkg.init_system()
    acquire = apt_pkg.Acquire()
    slist = apt_pkg.SourceList()
    # Read the list
    slist.read_main_list()
    # Add all indexes to the fetcher.
    slist.get_indexes(acquire, True)

    # Now print the URI of every item.
    for item in acquire.items:
        print(item.desc_uri)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
#
# usage: ./utils/expand_info.py build/data/templates/Debian.info \
#                               /usr/share/distro-info/debian.csv

import sys

import aptsources.distinfo

for line in aptsources.distinfo._expand_template(sys.argv[1], sys.argv[2]):
    print(line)



#!/usr/bin/python3

import apt

cache = apt.Cache()

for pkg in cache:
    if not pkg.candidate.record:
        continue
    if "Task" in pkg.candidate.record:
        print(
            "Pkg {} is part of '{}'".format(
                pkg.name, pkg.candidate.record["Task"].split()
            )
        )
        # print pkg.candidateRecord



#!/usr/bin/python3
#
# Copyright (C) 2012 Michael Vogt <mvo@ubuntu.com>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.

import unittest

import apt_pkg
import testcommon

import apt


class TestAptPolicy(testcommon.TestCase):
    def test_apt_policy_lowlevel(self):
        return  # TODO: Make tests independent of system state
        # get a policy
        cache = apt.Cache()
        policy = cache._depcache.policy
        self.assertNotEqual(policy, None)
        # basic tests
        pkg = cache["apt"]
        self.assertEqual(policy.get_priority(pkg._pkg), 0)
        # get priority for all pkgfiles
        for ver in pkg.versions:
            lowlevel_ver = ver._cand
            for pkgfile, i in lowlevel_ver.file_list:
                # print pkgfile, i, policy.get_priority(pkgfile)
                self.assertTrue(policy.get_priority(pkgfile) >= 1)
                self.assertTrue(policy.get_priority(pkgfile) < 1001)

    def test_apt_policy_lowlevel_files(self):
        cache = apt_pkg.Cache()
        depcache = apt_pkg.DepCache(cache)
        policy = cache.policy
        dpolicy = depcache.policy
        self.assertNotEqual(policy, None)
        self.assertNotEqual(dpolicy, None)

        for f in cache.file_list:
            policy.get_priority(f)
            dpolicy.get_priority(f)

    def test_apt_policy_lowlevel_versions(self):
        cache = apt_pkg.Cache()
        depcache = apt_pkg.DepCache(cache)
        policy = cache.policy
        dpolicy = depcache.policy
        self.assertNotEqual(policy, None)
        self.assertNotEqual(dpolicy, None)

        for pkg in cache.packages:
            for ver in pkg.version_list:
                policy.get_priority(ver)
                dpolicy.get_priority(ver)

    def test_apt_policy_highlevel(self):
        return  # TODO: Make tests independent of system state
        cache = apt.Cache()
        pkg = cache["apt"]
        self.assertTrue(
            pkg.candidate.policy_priority > 1 and pkg.candidate.policy_priority < 1001
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
#
# Copyright (C) 2010 Michael Vogt <mvo@ubuntu.com>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Unit tests for verifying the correctness of DebPackage in apt.debfile."""
import sys
import unittest

from test_all import get_library_dir

libdir = get_library_dir()
if libdir:
    sys.path.insert(0, libdir)
import apt_pkg
import testcommon

import apt
import apt.debfile


class TestDebfileMultiarch(testcommon.TestCase):
    """test the multiarch debfile"""

    def test_multiarch_deb_check(self):
        if apt_pkg.get_architectures() != ["amd64", "i386"]:
            # TODO: use unittest.skip
            # logging.warning("skipping test because running on a "
            #                "non-multiarch system")
            return
        deb = apt.debfile.DebPackage("./data/test_debs/multiarch-test1_i386.deb")
        deb.check()
        missing = deb.missing_deps
        # print missing
        self.assertFalse("dpkg:i386" in missing)

    @unittest.skip("BROKEN, lib3ds-1-3 is m-a now")
    def test_multiarch_conflicts(self):
        cache = apt.Cache()
        # WARNING: this assumes that lib3ds-1-3 is a non-multiarch lib
        # use "lib3ds-1-3" as a test to see if non-multiach lib conflicts work
        canary = "lib3ds-1-3"
        if canary not in cache:
            # TODO: use unittest.skip
            # logging.warning("skipping test because %s is missing" % canary)
            return
        cache[canary].mark_install()
        deb = apt.debfile.DebPackage(
            "./data/test_debs/multiarch-test1_i386.deb", cache=cache
        )
        # this deb should now not be installable
        installable = deb.check()
        # print deb._failure_string
        self.assertFalse(installable)
        self.assertEqual(
            deb._failure_string, "Conflicts with the installed package 'lib3ds-1-3'"
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
import apt_pkg


def main():
    """Example for PackageFile()"""
    apt_pkg.init()
    cache = apt_pkg.Cache()
    for pkgfile in cache.file_list:
        print("Package-File:", pkgfile.filename)
        print("Index-Type:", pkgfile.index_type)  # 'Debian Package Index'
        if pkgfile.not_source:
            print("Source: None")
        else:
            if pkgfile.site:
                # There is a source, and a site, print the site
                print("Source:", pkgfile.site)
            else:
                # It seems to be a local repository
                print("Source: Local package file")
        if pkgfile.not_automatic:
            # The system won't be updated automatically (eg. experimental)
            print("Automatic: No")
        else:
            print("Automatic: Yes")
        print()


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# Example demonstrating how to use the configuration/commandline system
# for configuration.
# Some valid command lines..
#   config.py -h --help            ; Turn on help
#   config.py -no-h --no-help --help=no  ; Turn off help
#   config.py -qqq -q=3            ; verbosity to 3
#   config.py -c /etc/apt/apt.conf ; include that config file]
#   config.py -o help=true         ; Turn on help by giving a
#                                  ; config file string
#   config.py -no-h -- -help       ; Turn off help, specify the file '-help'
# -c and -o are standard APT-program options.

import posixpath
import sys

# This shows how to use the system for configuration and option control.
# The other varient is for ISC object config files. See configisc.py.
import apt_pkg

# Create a new empty Configuration object - there is also the system global
# configuration object apt_pkg.config which is used interally by apt-pkg
# routines to control unusual situations. I recommend using the sytem global
# whenever possible..
Cnf = apt_pkg.Configuration()

print("Command line is", sys.argv)

# Load the default configuration file, init_config() does this better..
Cnf.set("config-file", "/etc/apt/apt.conf")  # or Cnf["config-file"] = ".."
if posixpath.exists(Cnf.find_file("config-file")):
    apt_pkg.read_config_file(Cnf, "/etc/apt/apt.conf")

# Merge the command line arguments into the configuration space
Arguments = [
    ("h", "help", "help"),
    ("v", "version", "version"),
    ("q", "quiet", "quiet", "IntLevel"),
    ("c", "config-file", "", "ConfigFile"),
    ("o", "option", "", "ArbItem"),
]
print("FileNames", apt_pkg.parse_commandline(Cnf, Arguments, sys.argv))

print("Quiet level selected is", Cnf.find_i("quiet", 0))

# Do some stuff with it
if Cnf.find_b("version", 0) == 1:
    print("Version selected - 1.1")

if Cnf.find_b("help", 0) == 1:
    print("python-apt", apt_pkg.VERSION, "compiled on", apt_pkg.DATE, apt_pkg.TIME)
    print("Hi, I am the help text for this program")
    sys.exit(0)

print("No help for you, try -h")

# Print the configuration space
print("The Configuration space looks like:")
for item in list(Cnf.keys()):
    print(f'{item} "{Cnf[item]}";')



#!/usr/bin/python3
import sys

import apt_pkg

import apt


def show_help():
    print(f"apt {apt_pkg.VERSION} compiled on {apt_pkg.DATE} {apt_pkg.TIME}")
    if apt_pkg.config.find_b("version"):
        return 0

    # Copied from apt-cdrom
    print(
        "Usage: apt-cdrom [options] command\n"
        "\n"
        "apt-cdrom is a tool to add CDROM's to APT's source list. The\n"
        "CDROM mount point and device information is taken from apt.conf\n"
        "and /etc/fstab.\n"
        "\n"
        "Commands:\n"
        "   add - Add a CDROM\n"
        "   ident - Report the identity of a CDROM\n"
        "\n"
        "Options:\n"
        "  -h   This help text\n"
        "  -d   CD-ROM mount point\n"
        "  -r   Rename a recognized CD-ROM\n"
        "  -m   No mounting\n"
        "  -f   Fast mode, don't check package files\n"
        "  -a   Thorough scan mode\n"
        "  -c=? Read this configuration file\n"
        "  -o=? Set an arbitrary configuration option, eg -o "
        "dir::cache=/tmp\n"
        "See fstab(5)"
    )
    return 0


def main(args):
    arguments = apt_pkg.parse_commandline(
        apt_pkg.config,
        [
            ("h", "help", "help"),
            ("v", "version", "version"),
            ("d", "cdrom", "Acquire::cdrom::mount", "HasArg"),
            ("r", "rename", "APT::CDROM::Rename"),
            ("m", "no-mount", "APT::CDROM::NoMount"),
            ("f", "fast", "APT::CDROM::Fast"),
            ("n", "just-print", "APT::CDROM::NoAct"),
            ("n", "recon", "APT::CDROM::NoAct"),
            ("n", "no-act", "APT::CDROM::NoAct"),
            ("a", "thorough", "APT::CDROM::Thorough"),
            ("c", "config-file", "", "ConfigFile"),
            ("o", "option", "", "ArbItem"),
        ],
        args,
    )

    if apt_pkg.config.find_b("help") or apt_pkg.config.find_b("version"):
        return show_help()

    progress = apt.progress.text.CdromProgress()
    cdrom = apt_pkg.Cdrom()

    if not arguments:
        return show_help()
    elif arguments[0] == "add":
        cdrom.add(progress)
    elif arguments[0] == "ident":
        cdrom.ident(progress)
    else:
        sys.stderr.write("E: Invalid operation %s\n" % arguments[0])
        return 1


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/python3
import unittest

import apt_pkg
import testcommon


class TestLargeFileSupport(testcommon.TestCase):
    """Test large file support"""

    def test_acquire_file(self):
        """Test apt_pkg.AcquireFile() accepts large file size"""
        acq = apt_pkg.Acquire()
        fil = apt_pkg.AcquireFile(acq, "http://foo", "foo", size=2875204834)
        self.assertEqual(fil.filesize, 2875204834)


if __name__ == "__main__":
    unittest.main()



#
# Test that both unicode and bytes path names work
#
import os
import shutil
import unittest

import apt_inst
import apt_pkg
import testcommon


class TestPath(testcommon.TestCase):
    dir_unicode = "data/tmp"
    dir_bytes = b"data/tmp"
    file_unicode = "data/tmp/python-apt-test"
    file_bytes = b"data/tmp/python-apt-test"

    def setUp(self):
        testcommon.TestCase.setUp(self)
        if os.path.exists(self.dir_bytes):
            shutil.rmtree(self.dir_bytes)

        os.mkdir(self.dir_bytes)

    def tearDown(self):
        apt_pkg.config["dir"] = "/"
        shutil.rmtree(self.dir_bytes)

    def test_acquire(self):
        apt_pkg.AcquireFile(
            apt_pkg.Acquire(),
            "http://example.com",
            destdir=self.file_bytes,
            destfile=self.file_bytes,
        )
        apt_pkg.AcquireFile(
            apt_pkg.Acquire(),
            "http://example.com",
            destdir=self.file_unicode,
            destfile=self.file_unicode,
        )

    def test_acquire_hashes(self):
        hs = apt_pkg.HashString("d41d8cd98f00b204e9800998ecf8427e")
        hsl = apt_pkg.HashStringList()
        hsl.append(hs)
        apt_pkg.AcquireFile(
            apt_pkg.Acquire(),
            "http://example.com",
            hash=hsl,
            destdir=self.file_unicode,
            destfile=self.file_unicode,
        )
        apt_pkg.AcquireFile(
            apt_pkg.Acquire(),
            "http://example.com",
            hash=str(hs),
            destdir=self.file_unicode,
            destfile=self.file_unicode,
        )
        self.assertRaises(
            TypeError,
            apt_pkg.AcquireFile,
            apt_pkg.Acquire(),
            "http://example.com",
            hash=hs,
            destdir=self.file_unicode,
            destfile=self.file_unicode,
        )

    def test_ararchive(self):
        archive = apt_inst.ArArchive("data/test_debs/data-tar-xz.deb")

        apt_inst.ArArchive(b"data/test_debs/data-tar-xz.deb")

        archive.extract("debian-binary", "data/tmp")
        archive.extract(b"debian-binary", b"data/tmp")
        archive.extractall("data/tmp")
        archive.extractall(b"data/tmp")
        self.assertEqual(archive.extractdata("debian-binary"), b"2.0\n")
        self.assertEqual(archive.extractdata(b"debian-binary"), b"2.0\n")
        self.assertTrue(archive.getmember("debian-binary"))
        self.assertTrue(archive.getmember(b"debian-binary"))
        self.assertTrue("debian-binary" in archive)
        self.assertTrue(b"debian-binary" in archive)
        self.assertTrue(archive[b"debian-binary"])
        self.assertTrue(archive["debian-binary"])

        tar = archive.gettar("control.tar.xz", "xz")
        tar = archive.gettar(b"control.tar.xz", "xz")

        tar.extractall(self.dir_unicode)
        tar.extractall(self.dir_bytes)
        self.assertRaises(LookupError, tar.extractdata, "Do-not-exist")
        self.assertRaises(LookupError, tar.extractdata, b"Do-not-exist")
        tar.extractdata(b"control")
        tar.extractdata("control")

        apt_inst.TarFile(os.path.join(self.dir_unicode, "control.tar.xz"))
        apt_inst.TarFile(os.path.join(self.dir_bytes, b"control.tar.xz"))

    def test_configuration(self):
        with open(self.file_unicode, "w") as config:
            config.write("Hello { World 1; };")
        apt_pkg.read_config_file(apt_pkg.config, self.file_bytes)
        apt_pkg.read_config_file(apt_pkg.config, self.file_unicode)
        apt_pkg.read_config_file_isc(apt_pkg.config, self.file_bytes)
        apt_pkg.read_config_file_isc(apt_pkg.config, self.file_unicode)
        apt_pkg.read_config_dir(apt_pkg.config, self.dir_unicode)
        apt_pkg.read_config_dir(apt_pkg.config, b"/etc/apt/apt.conf.d")

    def test_index_file(self):
        apt_pkg.config["dir"] = "data/test_debs"
        slist = apt_pkg.SourceList()
        slist.read_main_list()

        for meta in slist.list:
            for index in meta.index_files:
                index.archive_uri(self.file_bytes)
                index.archive_uri(self.file_unicode)

    def test_lock(self):
        apt_pkg.get_lock(self.file_unicode, True)
        apt_pkg.get_lock(self.file_bytes, True)

        with apt_pkg.FileLock(self.file_unicode):
            pass
        with apt_pkg.FileLock(self.file_bytes):
            pass

    def test_policy(self):
        apt_pkg.config["dir"] = "data/test_debs"
        cache = apt_pkg.Cache(None)
        policy = apt_pkg.Policy(cache)
        file_unicode = os.path.join(self.dir_unicode, "test.prefs")
        file_bytes = os.path.join(self.dir_bytes, b"test.prefs")

        self.assertTrue(policy.read_pinfile(file_unicode))
        self.assertTrue(policy.read_pinfile(file_bytes))
        self.assertTrue(policy.read_pindir(self.dir_unicode))
        self.assertTrue(policy.read_pindir(self.dir_bytes))

    def test_tag(self):
        with open(self.file_bytes, "w") as tag:
            tag.write("Key: value\n")
        tag1 = apt_pkg.TagFile(self.file_unicode)
        tag2 = apt_pkg.TagFile(self.file_bytes)

        self.assertEqual(next(tag1)["Key"], "value")
        self.assertEqual(next(tag2)["Key"], "value")

        self.assertRaises(StopIteration, next, tag1)
        self.assertRaises(StopIteration, next, tag2)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

import sys
import time

import apt_pkg

import apt
import apt.progress.base


class TextProgress(apt.progress.base.OpProgress):
    def __init__(self):
        self.last = 0.0

    def update(self, percent):
        if (self.last + 1.0) <= percent:
            sys.stdout.write("\rProgress: %i.2          " % (percent))
            self.last = percent
        if percent >= 100:
            self.last = 0.0

    def done(self):
        self.last = 0.0
        print("\rDone                      ")


class TextFetchProgress(apt.progress.base.AcquireProgress):
    def __init__(self):
        pass

    def start(self):
        pass

    def stop(self):
        pass

    def fail(self, item):
        print("fail", item)

    def fetch(self, item):
        print("fetch", item)

    def ims_hit(self, item):
        print("ims_hit", item)

    def pulse(self, owner):
        print(
            "pulse: CPS: %s/s; Bytes: %s/%s; Item: %s/%s"
            % (
                apt_pkg.size_to_str(self.current_cps),
                apt_pkg.size_to_str(self.current_bytes),
                apt_pkg.size_to_str(self.total_bytes),
                self.current_items,
                self.total_items,
            )
        )
        return True

    def media_change(self, medium, drive):
        print(f"Please insert medium {medium} in drive {drive}")
        sys.stdin.readline()
        # return False


class TextInstallProgress(apt.progress.base.InstallProgress):
    def __init__(self):
        apt.progress.base.InstallProgress.__init__(self)

    def start_update(self):
        print("start_update")

    def finish_update(self):
        print("finish_update")

    def status_change(self, pkg, percent, status):
        print(f"[{percent}] {pkg}: {status}")

    def update_interface(self):
        apt.progress.base.InstallProgress.update_interface(self)
        # usefull to e.g. redraw a GUI
        time.sleep(0.1)


class TextCdromProgress(apt.progress.base.CdromProgress):
    def __init__(self):
        pass

    # update is called regularly so that the gui can be redrawn

    def update(self, text, step):
        # check if we actually have some text to display
        if text != "":
            print(f"Update: {text.strip()} {step}")

    def ask_cdrom_name(self):
        sys.stdout.write("Please enter cd-name: ")
        cd_name = sys.stdin.readline()
        return (True, cd_name.strip())

    def change_cdrom(self):
        print("Please insert cdrom and press <ENTER>")
        answer = sys.stdin.readline()
        print(answer)
        return True


if __name__ == "__main__":
    c = apt.Cache()
    pkg = c["3dchess"]
    if pkg.is_installed:
        pkg.mark_delete()
    else:
        pkg.mark_install()

    res = c.commit(TextFetchProgress(), TextInstallProgress())

    print(res)



#!/usr/bin/python3
# example how to deal with the depcache

import sys

import apt_pkg
from progress import TextCdromProgress

# init
apt_pkg.init()

cdrom = apt_pkg.Cdrom()
print(cdrom)

progress = TextCdromProgress()

(res, ident) = cdrom.ident(progress)
print(f"ident result is: {res} ({ident}) ")

apt_pkg.config["APT::CDROM::Rename"] = "True"
cdrom.add(progress)

print("Exiting")
sys.exit(0)



#!/usr/bin/python3
import os

import apt_pkg

import apt
import apt.progress.text


def get_file(fetcher, uri, destfile):
    # get the file
    af = apt_pkg.AcquireFile(fetcher, uri=uri, descr="sample descr", destfile=destfile)
    print(f"desc_uri: {af.desc_uri} -> {af.destfile}")
    res = fetcher.run()
    if res != fetcher.RESULT_CONTINUE:
        return False
    return True


apt_pkg.init()

# apt_pkg.config.set("Debug::pkgDPkgPM","1");
# apt_pkg.config.set("Debug::pkgPackageManager","1");
# apt_pkg.config.set("Debug::pkgDPkgProgressReporting","1");

cache = apt_pkg.Cache()
depcache = apt_pkg.DepCache(cache)

recs = apt_pkg.PackageRecords(cache)
list = apt_pkg.SourceList()
list.read_main_list()

# show the amount fetch needed for a dist-upgrade
depcache.upgrade(True)
progress = apt.progress.text.AcquireProgress()
fetcher = apt_pkg.Acquire(progress)
pm = apt_pkg.PackageManager(depcache)
pm.get_archives(fetcher, list, recs)
print(f"{apt_pkg.size_to_str(fetcher.fetch_needed)} ({fetcher.fetch_needed})")
actiongroup = apt_pkg.ActionGroup(depcache)
for pkg in cache.packages:
    depcache.mark_keep(pkg)

try:
    os.mkdir("/tmp/pyapt-test")
    os.mkdir("/tmp/pyapt-test/partial")
except OSError:
    pass
apt_pkg.config.set("Dir::Cache::archives", "/tmp/pyapt-test")

pkg = cache["2vcard"]
depcache.mark_install(pkg)

progress = apt.progress.text.AcquireProgress()
fetcher = apt_pkg.Acquire(progress)
# fetcher = apt_pkg.Acquire()
pm = apt_pkg.PackageManager(depcache)

print(pm)
print(fetcher)

get_file(fetcher, "http://ftp.debian.org/debian/dists/README", "/tmp/lala")

pm.get_archives(fetcher, list, recs)

for item in fetcher.items:
    print(item)
    if item.status == item.STAT_ERROR:
        print("Some error ocured: '%s'" % item.error_text)
    if not item.complete:
        print("No error, still nothing downloaded (%s)" % item.error_text)
    print()


res = fetcher.run()
print("fetcher.Run() returned: %s" % res)

print("now runing pm.DoInstall()")
progress = apt_pkg.PackageManagerProgressFancy()
res = pm.do_install(progress=progress)
print("pm.DoInstall() returned: %s" % res)



#!/usr/bin/python3
# Builds on python2.X and python3
# $Id: setup.py,v 1.2 2002/01/08 07:13:21 jgg Exp $
import glob
import os
import shutil
import subprocess
import sys

from setuptools import Extension, setup
from setuptools.command.install import install

cmdclass = {}

try:
    from DistUtilsExtra.auto import clean_build_tree
    from DistUtilsExtra.command import build_extra, build_i18n
except ImportError:
    print("W: [python%s] DistUtilsExtra import error." % sys.version[:3])
else:
    cmdclass["build"] = build_extra.build_extra
    cmdclass["build_i18n"] = build_i18n.build_i18n
    cmdclass["clean"] = clean_build_tree


class InstallTypeinfo(install):
    def run(self):
        install.run(self)
        for pyi in glob.glob("typehinting/*.pyi"):
            stubs = os.path.basename(pyi).split(".")[0] + "-stubs"
            stubs = os.path.join(self.install_purelib, stubs)
            if not os.path.exists(stubs):
                os.makedirs(stubs)
            shutil.copy(pyi, os.path.join(stubs, "__init__.pyi"))


cmdclass["install"] = InstallTypeinfo


def get_version():
    """Get a PEP 0440 compatible version string"""
    version = os.environ.get("DEBVER")
    if not version:
        proc = subprocess.Popen(
            ["dpkg-parsechangelog", "-SVersion"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = proc.communicate()
        if proc.returncode == 0:
            version = out.decode("utf-8")

    version = version.strip()

    if not version:
        return version

    version = version.replace("~alpha", ".a")
    version = version.replace("~beta", ".b")
    version = version.replace("~rc", ".rc")
    version = version.replace("~exp", ".dev")
    version = version.replace("ubuntu", "+ubuntu")
    version = version.replace("tanglu", "+tanglu")
    version = version.split("build")[0]

    return version


# The apt_pkg module.
files = [
    "apt_pkgmodule.cc",
    "acquire.cc",
    "cache.cc",
    "cdrom.cc",
    "configuration.cc",
    "depcache.cc",
    "generic.cc",
    "hashes.cc",
    "hashstring.cc",
    "indexfile.cc",
    "metaindex.cc",
    "pkgmanager.cc",
    "pkgmanagerprogress.cc",
    "pkgrecords.cc",
    "pkgsrcrecords.cc",
    "policy.cc",
    "progress.cc",
    "sourcelist.cc",
    "string.cc",
    "tag.cc",
    "lock.cc",
    "acquire-item.cc",
    "python-apt-helpers.cc",
    "cachegroup.cc",
    "orderlist.cc",
    "hashstringlist.cc",
]
files = sorted(["python/" + fname for fname in files], key=lambda s: s[:-3])
apt_pkg = Extension(
    "apt_pkg",
    files,
    # Hack around resolution order for duplicate versioned symbols so we prefer the stdc++ ones
    libraries=["stdc++", "apt-pkg"],
    extra_compile_args=[
        "-Wno-write-strings",
        "-DAPT_8_CLEANER_HEADERS",
        "-DAPT_9_CLEANER_HEADERS",
        "-DAPT_10_CLEANER_HEADERS",
        "-DPY_SSIZE_T_CLEAN",
    ],
)

# The apt_inst module
files = [
    "python/apt_instmodule.cc",
    "python/generic.cc",
    "python/arfile.cc",
    "python/tarfile.cc",
]
apt_inst = Extension(
    "apt_inst",
    files,
    libraries=["apt-pkg"],
    extra_compile_args=["-Wno-write-strings", "-DPY_SSIZE_T_CLEAN"],
)

# Replace the leading _ that is used in the templates for translation
if len(sys.argv) > 1 and sys.argv[1] == "build":
    if not os.path.exists("build/data/templates/"):
        os.makedirs("build/data/templates")
    for template in glob.glob("data/templates/*.info.in"):
        source = open(template)
        build = open("build/" + template[:-3], "w")
        for line in source:
            build.write(line.lstrip("_"))
        source.close()
        build.close()
    for template in glob.glob("data/templates/*.mirrors"):
        shutil.copy(template, os.path.join("build", template))


setup(
    name="python-apt",
    description="Python bindings for APT",
    version=get_version(),
    author="APT Development Team",
    author_email="deity@lists.debian.org",
    ext_modules=[apt_pkg, apt_inst],
    packages=["apt", "apt.progress", "aptsources"],
    package_data={
        "apt": ["*.pyi", "py.typed"],
    },
    data_files=[
        ("share/python-apt/templates", glob.glob("build/data/templates/*.info")),
        ("share/python-apt/templates", glob.glob("data/templates/*.mirrors")),
    ],
    cmdclass=cmdclass,
    license="GNU GPL",
    platforms="posix",
)



#!/usr/bin/python3
"""Example for packages. Print all essential and important packages"""

import apt_pkg


def main():
    """Main."""
    apt_pkg.init_config()
    apt_pkg.init_system()
    cache = apt_pkg.Cache()
    print("Essential packages:")
    for pkg in cache.packages:
        if pkg.essential:
            print(" ", pkg.name)
    print("Important packages:")
    for pkg in cache.packages:
        if pkg.important:
            print(" ", pkg.name)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# this is a example how to access the build dependencies of a package

import sys

import apt_pkg


def get_source_pkg(pkg, records, depcache):
    """get the source package name of a given package"""
    version = depcache.get_candidate_ver(pkg)
    if not version:
        return None
    file, index = version.file_list.pop(0)
    records.lookup((file, index))
    if records.source_pkg != "":
        srcpkg = records.source_pkg
    else:
        srcpkg = pkg.name
    return srcpkg


# main
apt_pkg.init()
cache = apt_pkg.Cache()
depcache = apt_pkg.DepCache(cache)
depcache.init()
records = apt_pkg.PackageRecords(cache)
srcrecords = apt_pkg.SourceRecords()

# base package that we use for build-depends calculation
if len(sys.argv) < 2:
    print("need a package name as argument")
    sys.exit(1)
try:
    pkg = base = cache[sys.argv[1]]
except KeyError:
    print("No package %s found" % sys.argv[1])
    sys.exit(1)
all_build_depends = set()

# get the build depdends for the package itself
srcpkg_name = get_source_pkg(base, records, depcache)
print("srcpkg_name: %s " % srcpkg_name)
if not srcpkg_name:
    print("Can't find source package for '%s'" % pkg.mame)
srcrec = srcrecords.lookup(srcpkg_name)
if srcrec:
    print("Files:")
    print(srcrecords.files)
    bd = srcrecords.build_depends
    print("build-depends of the package: %s " % bd)
    for b in bd:
        all_build_depends.add(b[0])

# calculate the build depends for all dependencies
depends = depcache.get_candidate_ver(base).depends_list
for dep in depends["Depends"]:  # FIXME: do we need to consider PreDepends?
    pkg = dep[0].target_pkg
    srcpkg_name = get_source_pkg(pkg, records, depcache)
    if not srcpkg_name:
        print("Can't find source package for '%s'" % pkg.name)
        continue
    srcrec = srcrecords.lookup(srcpkg_name)
    if srcrec:
        # print srcrecords.package
        # print srcrecords.binaries
        bd = srcrecords.build_depends
        # print "%s: %s " % (srcpkg_name, bd)
        for b in bd:
            all_build_depends.add(b[0])


print("\n".join(all_build_depends))



"""Common testing stuff"""

import os
import unittest

import apt_pkg


class TestCase(unittest.TestCase):
    """Base class for python-apt unittests"""

    def setUp(self):
        self.resetConfig()
        os.chdir(os.path.dirname(__file__))

    def resetConfig(self):
        apt_pkg.config.clear("")
        for key in apt_pkg.config.list():
            apt_pkg.config.clear(key)

        # Avoid loading any host config files
        os.unsetenv("APT_CONFIG")
        apt_pkg.config["Dir::Etc::main"] = "/dev/null"
        apt_pkg.config["Dir::Etc::parts"] = "/dev/null"
        apt_pkg.config["APT::Sandbox::User"] = "root"

        apt_pkg.init_config()
        apt_pkg.init_system()

        # Restore default values
        apt_pkg.config["Dir::Etc::main"] = "apt.conf"
        apt_pkg.config["Dir::Etc::parts"] = "apt.conf.d"



#!/usr/bin/python-dbg

import gc
import sys
from pprint import pprint

import apt

# get initial cache
print(sys.gettotalrefcount())
progress = apt.progress.OpTextProgress()
c = apt.Cache(progress)
print("refcount after first cache instance: ", sys.gettotalrefcount())

# test open()
c.open(progress)
print("refcount after cache open: ", sys.gettotalrefcount())
# pprint(sys.getobjects(10))

c.open(apt.progress.OpProgress())
print("refcount after seconf cache open: ", sys.gettotalrefcount())
# pprint(sys.getobjects(10))

# FIXME: find a way to get a efficient diff
# before = gc.get_objects()
# c.open(apt.progress.OpProgress())
# after = gc.get_objects()


# test update()
print("refcount before cache.update(): ", sys.gettotalrefcount())
c.update()
gc.collect()
print("refcount after cache.update(): ", sys.gettotalrefcount())
c.update()
gc.collect()
print("refcount after second cache.update(): ", sys.gettotalrefcount())
# pprint(sys.getobjects(20))


# test install()
c.open(apt.progress.OpProgress())
gc.collect()
print("refcount before cache['hello'].mark_install(): ", sys.gettotalrefcount())
c["hello"].mark_install()
c.commit(apt.progress.FetchProgress(), apt.progress.InstallProgress())
gc.collect()
print("refcount after: ", sys.gettotalrefcount())
c.open(apt.progress.OpProgress())
c["hello"].mark_delete()
c.commit(apt.progress.FetchProgress(), apt.progress.InstallProgress())
gc.collect()
print("refcount after: ", sys.gettotalrefcount())
pprint(sys.getobjects(10))



#!/usr/bin/python3

import apt

cache = apt.Cache()

for pkg in cache:
    if pkg.installed and pkg.installed.enhances:
        s = "%s enhances:" % pkg.name
        for or_list in pkg.installed.enhances:
            for enhances in or_list.or_dependencies:
                s += " %s" % enhances.name
                if enhances.name in cache and not cache[enhances.name].is_installed:
                    s += "(*missing*) "
                s += ","
            print(s[:-1])



#!/usr/bin/python3
#
# Test for the pkgCache code
#

import sys

import apt_pkg


def main():
    apt_pkg.init()
    cache = apt_pkg.Cache()
    depcache = apt_pkg.DepCache(cache)
    depcache.init()
    i = 0
    all = cache.package_count
    print("Running Cache test on all packages:")
    # first, get all pkgs
    for pkg in cache.packages:
        i += 1
        pkg.name
        # then get each version
        for ver in pkg.version_list:
            # get some version information
            ver.file_list
            ver.ver_str
            ver.arch
            ver.depends_listStr
            dl = ver.depends_list
            # get all dependencies (a dict of string->list,
            # e.g. "depends:" -> [ver1,ver2,..]
            for dep in dl.keys():
                # get the list of each dependency object
                for depVerList in dl[dep]:
                    for z in depVerList:
                        # get all TargetVersions of
                        # the dependency object
                        for j in z.all_targets():
                            j.file_list
                            ver.ver_str
                            ver.arch
                            ver.depends_listStr
                            j = ver.depends_list

        print("\r%i/%i=%.3f%%    " % (i, all, (float(i) / float(all) * 100)))


if __name__ == "__main__":
    main()
    sys.exit(0)



#!/usr/bin/python3
#
#
# this example is not usefull to find out about updated, upgradable packages
# use the depcache.py example for it (because a pkgPolicy is not used here)
#

import apt_pkg

apt_pkg.init()

cache = apt_pkg.Cache()
packages = cache.packages

uninstalled, updated, upgradable = {}, {}, {}

for package in packages:
    versions = package.version_list
    if not versions:
        continue
    version = versions[0]
    for other_version in versions:
        if apt_pkg.version_compare(version.ver_str, other_version.ver_str) < 0:
            version = other_version
    if package.current_ver:
        current = package.current_ver
        if apt_pkg.version_compare(current.ver_str, version.ver_str) < 0:
            upgradable[package.name] = version
            break
        else:
            updated[package.name] = current
    else:
        uninstalled[package.name] = version


for line in (uninstalled, updated, upgradable):
    print(list(line.items())[0])
